@@ -1,12 +1,15 @@
 /*
-  Hatari - cartdata.c
+  Hatari - cartData.c
 
   This file is distributed under the GNU Public License, version 2 or at
   your option any later version. Read the file gpl.txt for details.
 
-  The assembled cartridge Pexec program (source code can be found in cart_asm.s)
+  The assembled cartridge Pexec() program (source code can be found in the
+  file cart_asm.s).
+  NOTE: This file is included by cart.c - do not compile and link this file
+  separately!
 */
-char CartData_rcsid[] = "Hatari $Id: cartData.c,v 1.2 2005/03/17 09:01:17 thothy Exp $";
+const char CartData_rcsid[] = "Hatari $Id: cartData.c,v 1.5 2006/02/12 21:28:22 eerot Exp $";
 
 
 /*
@@ -15,7 +18,7 @@ char CartData_rcsid[] = "Hatari $Id: cartData.c,v 1.2 2005/03/17 09:01:17 thothy
   file or use an assembler like TurboAss that can generate absolute binary images.
   (I am using TurboAss and  hexdump -v -e ' 16/1 "0x%02x," "\n" '  to create it).
  */
-Uint8 Cart_data[] =
+const Uint8 Cart_data[] =
 {
 0xab,0xcd,0xef,0x42,0x00,0x00,0x00,0x00,0x08,0xfa,0x02,0x04,0x00,0xfa,0x02,0x0a,
 0x58,0x00,0x32,0x29,0x00,0x00,0x36,0x9e,0x48,0x41,0x54,0x41,0x52,0x49,0x2e,0x54,
