@@ -71,6 +71,6 @@ static void ConvertLowRes_320x8Bit(void)
     }
     while(--x);              /* Loop on X */
 
-    pPCScreenDest = (void *)(((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);  /* Offset to next line */
+    pPCScreenDest = (((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);  /* Offset to next line */
   }
 }
