@@ -63,7 +63,7 @@ static void ConvertVDIRes_16Colour(void)
     while( --x );                    /* Loop on X */
 
     /* Offset to next line */
-    pPCScreenDest = (void *)(((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);
+    pPCScreenDest = (((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);
   }
 }
 
