@@ -56,6 +56,6 @@ static void ConvertVDIRes_4Colour(void)
     while( --x );                  /* Loop on X */
 
     /* Offset to next line */
-    pPCScreenDest = (void *)(((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);
+    pPCScreenDest = (((Uint8 *)pPCScreenDest)+PCScreenBytesPerLine);
   }
 }
