@@ -12,7 +12,7 @@
   checked each HBL to perform the transfer of data from our disk image into
   the ST RAM area by simulating the DMA.
 */
-char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.23 2005/09/26 15:20:14 thothy Exp $";
+const char FDC_rcsid[] = "Hatari $Id: fdc.c,v 1.30 2006/08/13 23:33:32 thothy Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -157,6 +157,8 @@ static short int nReadWriteSectors;
 
 static Uint8 DMASectorWorkSpace[NUMBYTESPERSECTOR];             /* Workspace used to copy to/from for floppy DMA */
 
+static int nFdcDelayHbls;                                       /* Used to slow down FDC emulation */
+
 
 /*-----------------------------------------------------------------------*/
 /*
@@ -271,7 +273,7 @@ void FDC_ResetDMAStatus(void)
 
 	HDCSectorCount = 0;
 	HDCCommand.byteCount = 0;         /* Reset HDC command status */
-	HDCCommand.returnCode = 0xFF;
+	HDCCommand.returnCode = 0;
 }
 
 
@@ -297,7 +299,7 @@ void FDC_SetDMAStatus(BOOL bError)
 	/* Set zero sector count */
 
 	if (DMAModeControl_ff8606wr&0x08)         /* Get which sector count? */
-		DMAStatus_ff8606rd |= (HDCSectorCount)?0:0x2;         /* HDC */
+		DMAStatus_ff8606rd |= (HDCSectorCount)?0x2:0;         /* HDC */
 	else
 		DMAStatus_ff8606rd |= (FDCSectorCountRegister)?0x2:0; /* FDC */
 	/* Perhaps the DRQ should be set here */
@@ -313,7 +315,7 @@ void FDC_DmaStatus_ReadWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode on a normal ST */
-		M68000_BusError(IoAccessBaseAddress, 1);
+		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_READ);
 		return;
 	}
 
@@ -406,6 +408,27 @@ static void FDC_SetReadWriteParameters(int nSectors)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  ST program (or TOS) has read the MFP GPIP register to check if the FDC
+  is already done. Then we can skip the usual FDC waiting period!
+*/
+void FDC_GpipRead(void)
+{
+	static int nLastGpipBit;
+
+	if ((MFP_GPIP & 0x20) == nLastGpipBit)
+	{
+		if (!ConfigureParams.System.bSlowFDC)
+			nFdcDelayHbls = 0;
+	}
+	else
+	{
+		nLastGpipBit = MFP_GPIP & 0x20;
+	}
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Update floppy drive on each HBL (approx' 512 cycles)
@@ -415,14 +438,10 @@ void FDC_UpdateHBL(void)
 	/* Seems like some games/demos (e.g. Fantasia by Dune, Alien World, ...) don't
 	 * work if the FDC is too fast... so here's a quick-n-dirty hack to get them
 	 * working... Should be replaced with proper FDC timings one day! */
-	static int nDelayHBLs = 300;
-	if (ConfigureParams.System.bSlowFDC)
-	{
-		if (nDelayHBLs-- > 0)
-			return;
-		else
-			nDelayHBLs = 180;
-	}
+	if (nFdcDelayHbls-- > 0)
+		return;
+	else
+		nFdcDelayHbls = 180;
 
 	/* Do we have a DMA ready to copy? */
 	if (bDMAWaiting)
@@ -1118,23 +1137,26 @@ void FDC_DiskController_WriteWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode on a normal ST */
-		M68000_BusError(IoAccessBaseAddress, 0);
+		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_WRITE);
 		return;
 	}
 
+	M68000_WaitState(4);
+
 	DiskControllerWord_ff8604wr = IoMem_ReadWord(0xff8604);
 
 	HDC_WriteCommandPacket();                 /*  Handle HDC functions */
 
 	/* filter hdc commands */
-	if ((DMAModeControl_ff8606wr & 0x0018) == 8 && nPartitions == 0)
-		return;
+	if ((DMAModeControl_ff8606wr & 0x0018) == 8)
+		return;	
 
 	/* Are we trying to set the SectorCount? */
 	if (DMAModeControl_ff8606wr&0x10)         /* Bit 4 */
 		FDC_WriteSectorCountRegister();
 	else
-	{   /* Write to FDC registers */
+	{
+		/* Write to FDC registers */
 		switch(DMAModeControl_ff8606wr&0x6)
 		{   /* Bits 1,2 (A1,A0) */
 		 case 0x0:                            /* 0 0 - Command register */
@@ -1166,15 +1188,22 @@ void FDC_DiskControllerStatus_ReadWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode on a normal ST */
-		M68000_BusError(IoAccessBaseAddress, 1);
+		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_READ);
 		return;
 	}
 
-	if (DMAModeControl_ff8606wr == 0x08A)     /* HDC status reg selected? */
+	M68000_WaitState(4);
+
+	if ((DMAModeControl_ff8606wr & 0x18) == 0x08)     /* HDC status reg selected? */
 	{
 		/* return the HDC status reg */
 		DiskControllerByte = HDCCommand.returnCode;
 	}
+	else if ((DMAModeControl_ff8606wr & 0x18) == 0x18)  /* HDC sector counter??? */
+	{
+		Log_Printf(LOG_DEBUG, "*** Read HDC sector counter???\n");
+		DiskControllerByte = HDCSectorCount;
+	}
 	else
 	{
 		/* old FDC code */
@@ -1333,7 +1362,7 @@ void FDC_DmaModeControl_WriteWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode on a normal ST */
-		M68000_BusError(IoAccessBaseAddress, 0);
+		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_WRITE);
 		return;
 	}
 
