@@ -18,14 +18,18 @@
   * rmdir routine, can't remove dir with files in it. (another tos/unix difference)
   * Fix bugs, there are probably a few lurking around in here..
 */
-char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.43 2005/10/04 15:31:52 thothy Exp $";
+const char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.53 2006/07/22 15:49:23 thothy Exp $";
 
+#include <string.h>
+#include <strings.h>
 #include <sys/stat.h>
 #include <time.h>
-#include <dirent.h>
 #include <ctype.h>
 #include <unistd.h>
+
+#ifndef WIN32
 #include <glob.h>
+#endif
 
 #include "main.h"
 #include "cart.h"
@@ -35,11 +39,14 @@ char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.43 2005/10/04 15:31:52 thothy Ex
 #include "floppy.h"
 #include "hdc.h"
 #include "gemdos.h"
+#include "gemdos_defines.h"
+#include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "misc.h"
 #include "printer.h"
 #include "rs232.h"
+#include "scandir.h"
 #include "stMemory.h"
 #include "uae-cpu/hatari-glue.h"
 #include "uae-cpu/maccess.h"
@@ -47,16 +54,12 @@ char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.43 2005/10/04 15:31:52 thothy Ex
 
 // #define GEMDOS_VERBOSE
 // uncomment the following line to debug filename lookups on hd
-// #define FILE_DEBUG 1
+// #define GEMDOS_FILE_DEBUG 1
 
 #define ENABLE_SAVING             /* Turn on saving stuff */
 
 #define INVALID_HANDLE_VALUE -1
 
-#ifndef MAX_PATH
-#define MAX_PATH 256
-#endif
-
 /* GLOB_ONLYDIR is a GNU extension for the glob() function and not defined
  * on some systems. We should probably use something different for this
  * case, but at the moment it we simply define it as 0... */
@@ -65,15 +68,58 @@ char Gemdos_rcsid[] = "Hatari $Id: gemdos.c,v 1.43 2005/10/04 15:31:52 thothy Ex
 #define GLOB_ONLYDIR 0
 #endif
 
+/* Maximum supported length of a GEMDOS path: */
+#define MAX_GEMDOS_PATH 256
 
-/* structure with all the drive-specific data for our emulated drives */
+
+/* Have we re-directed GemDOS vector to our own routines yet? */
+BOOL bInitGemDOS;
+
+/* structure with all the drive-specific data for our emulated drives,
+ * used by GEMDOS_EMU_ON macro
+ */
 EMULATEDDRIVE **emudrives = NULL;
 
+#define  ISHARDDRIVE(Drive)  (Drive!=-1)
+
+/*
+  Disk Tranfer Address (DTA)
+*/
+#define TOS_NAMELEN  14
+
+typedef struct {
+  Uint8 index[2];
+  Uint8 magic[4];
+  char dta_pat[TOS_NAMELEN];
+  char dta_sattrib;
+  char dta_attrib;
+  Uint8 dta_time[2];
+  Uint8 dta_date[2];
+  Uint8 dta_size[4];
+  char dta_name[TOS_NAMELEN];
+} DTA;
+
+#define DTA_MAGIC_NUMBER  0x12983476
+#define MAX_DTAS_FILES    256      /* Must be ^2 */
+#define CALL_PEXEC_ROUTINE 3       /* Call our cartridge pexec routine */
+
+#define  BASE_FILEHANDLE     64    /* Our emulation handles - MUST not be valid TOS ones, but MUST be <256 */
+#define  MAX_FILE_HANDLES    32    /* We can allow 32 files open at once */
+
+/* 
+   DateTime structure used by TOS call $57 f_dattime 
+   Changed to fix potential problem with alignment.
+*/
+typedef struct {
+  Uint16 word1;
+  Uint16 word2;
+} DATETIME;
+
 typedef struct
 {
 	BOOL bUsed;
 	FILE *FileHandle;
-	char szActualName[MAX_PATH];        /* used by F_DATIME (0x57) */
+	char szActualName[MAX_GEMDOS_PATH];        /* used by F_DATIME (0x57) */
 } FILE_HANDLE;
 
 typedef struct
@@ -82,16 +128,15 @@ typedef struct
 	int  nentries;                      /* number of entries in fs directory */
 	int  centry;                        /* current entry # */
 	struct dirent **found;              /* legal files */
-	char path[MAX_PATH];                /* sfirst path */
+	char path[MAX_GEMDOS_PATH];                /* sfirst path */
 } INTERNAL_DTA;
 
-FILE_HANDLE  FileHandles[MAX_FILE_HANDLES];
-INTERNAL_DTA InternalDTAs[MAX_DTAS_FILES];
-int DTAIndex;                               /* Circular index into above */
-BOOL bInitGemDOS;                           /* Have we re-directed GemDOS vector to our own routines yet? */
-DTA *pDTA;                                  /* Our GEMDOS hard drive Disk Transfer Address structure */
-unsigned short int CurrentDrive;            /* Current drive (0=A,1=B,2=C etc...) */
-Uint32 act_pd;                              /* Used to get a pointer to the current basepage */
+static FILE_HANDLE  FileHandles[MAX_FILE_HANDLES];
+static INTERNAL_DTA InternalDTAs[MAX_DTAS_FILES];
+static int DTAIndex;        /* Circular index into above */
+static DTA *pDTA;           /* Our GEMDOS hard drive Disk Transfer Address structure */
+static Uint16 CurrentDrive; /* Current drive (0=A,1=B,2=C etc...) */
+static Uint32 act_pd;       /* Used to get a pointer to the current basepage */
 
 
 #ifdef GEMDOS_VERBOSE
@@ -191,6 +236,33 @@ static const char *pszGemDOSNames[] =
 
 
 
+/* Poor Windows does not have a glob() function... */
+#ifdef WIN32
+#warning sorry, no glob function on Windows
+
+typedef struct
+{
+    size_t gl_pathc;    /* Count of paths matched so far  */
+    char **gl_pathv;    /* List of matched pathnames.  */
+    size_t gl_offs;     /* Slots to reserve in `gl_pathv'.  */
+} glob_t;
+
+int glob(const char *pattern, int flags,
+	 int errfunc(const char *epath, int eerrno),
+         glob_t *pglob)
+{
+	return 0;
+}
+
+void globfree(glob_t *pglob)
+{
+}
+
+#define mkdir(name,mode) mkdir(name)
+
+#endif  /* WIN32 */
+
+
 
 /*-------------------------------------------------------*/
 /*
@@ -270,11 +342,11 @@ static unsigned char GemDOS_ConvertAttribute(mode_t mode)
 */
 static BOOL PopulateDTA(char *path, struct dirent *file)
 {
-	char tempstr[MAX_PATH];
+	char tempstr[MAX_GEMDOS_PATH];
 	struct stat filestat;
 	int n;
 
-	sprintf(tempstr, "%s/%s", path, file->d_name);
+	snprintf(tempstr, sizeof(tempstr), "%s/%s", path, file->d_name);
 	n = stat(tempstr, &filestat);
 	if (n != 0)
 		return FALSE;   /* return on error */
@@ -364,7 +436,8 @@ static void fsfirst_dirname(char *string, char *new)
 {
 	int i=0;
 
-	sprintf(new, string);
+	strcpy(new, string);
+
 	/* convert to front slashes. */
 	i=0;
 	while (new[i] != '\0')
@@ -381,9 +454,9 @@ static void fsfirst_dirname(char *string, char *new)
 	while (new[i] != '/')
 		i--; /* find last slash */
 	new[i] = '\0';
-
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   Parse directory mask, e.g. "*.*"
@@ -471,7 +544,7 @@ void GemDOS_InitDrives(void)
 	{
 		emudrives = malloc( MAX_HARDDRIVES*sizeof(EMULATEDDRIVE *) );
 		for(i=0; i<MAX_HARDDRIVES; i++)
-			emudrives[0] = malloc( sizeof(EMULATEDDRIVE) );
+			emudrives[i] = malloc( sizeof(EMULATEDDRIVE) );
 	}
 
 	for(i=0; i<MAX_HARDDRIVES; i++)
@@ -535,7 +608,7 @@ void GemDOS_MemorySnapShot_Capture(BOOL bSave)
 	MemorySnapShot_Store(&bInitGemDOS,sizeof(bInitGemDOS));
 	if (bSave)
 	{
-		Addr = (unsigned int)((Uint8 *)pDTA - STRam);
+		Addr = ((Uint8 *)pDTA - STRam);
 		MemorySnapShot_Store(&Addr,sizeof(Addr));
 	}
 	else
@@ -620,6 +693,7 @@ static int GemDOS_FindDriveNumber(char *pszFileName)
 static int GemDOS_IsFileNameAHardDrive(char *pszFileName)
 {
 	int DriveLetter;
+	int n;
 
 	/* Do we even have a hard-drive? */
 	if (GEMDOS_EMU_ON)
@@ -627,9 +701,11 @@ static int GemDOS_IsFileNameAHardDrive(char *pszFileName)
 		/* Find drive letter (as number) */
 		DriveLetter = GemDOS_FindDriveNumber(pszFileName);
 		/* add support for multiple drives here.. */
-		if (DriveLetter == emudrives[0]->hd_letter)
+		for (n=0; n<MAX_HARDDRIVES; n++) {
+  		  if (DriveLetter == emudrives[n]->hd_letter)
 			return DriveLetter;
 	}
+	}
 	/* Not a high-level redirected drive, let TOS handle it */
 	return -1;
 }
@@ -656,8 +732,13 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 {
 	char *s,*start;
 
+	/* Check for valid string */
 	if (pszFileName[0] == '\0')
-		return; /* check for valid string */
+		return;
+
+	/* Is it a valid hard drive? */
+	if (Drive < 2)
+		return;
 
 	/* case full filename "C:\foo\bar" */
 	s=pszDestName;
@@ -789,7 +870,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 					found = 1;
 				}
 			}
-#if FILE_DEBUG
+#if GEMDOS_FILE_DEBUG
 			if (!found)
 			{
 				/* It's often normal, the gem uses this to test for existence */
@@ -801,7 +882,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 		}
 	}
 
-#if FILE_DEBUG
+#if GEMDOS_FILE_DEBUG
 	fprintf(stderr,"conv %s -> %s\n",pszFileName,pszDestName);
 #endif
 }
@@ -815,15 +896,15 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName, char *ps
 #if 0
 static BOOL GemDOS_Cauxin(Uint32 Params)
 {
-	unsigned char Char;
+	unsigned char c;
 
 	/* Wait here until a character is ready */
 	while(!RS232_GetStatus())
 		;
 
 	/* And read character */
-	RS232_ReadBytes(&Char,1);
-	Regs[REG_D0] = Char;
+	RS232_ReadBytes(&c,1);
+	Regs[REG_D0] = c;
 
 	return TRUE;
 }
@@ -837,11 +918,11 @@ static BOOL GemDOS_Cauxin(Uint32 Params)
 #if 0
 static BOOL GemDOS_Cauxout(Uint32 Params)
 {
-	unsigned char Char;
+	unsigned char c;
 
 	/* Send character to RS232 */
-	Char = STMemory_ReadWord(Params+SIZE_WORD);
-	RS232_TransferBytesTo(&Char,1);
+	c = STMemory_ReadWord(Params+SIZE_WORD);
+	RS232_TransferBytesTo(&c, 1);
 
 	return TRUE;
 }
@@ -854,11 +935,11 @@ static BOOL GemDOS_Cauxout(Uint32 Params)
 */
 static BOOL GemDOS_Cprnout(Uint32 Params)
 {
-	unsigned char Char;
+	unsigned char c;
 
 	/* Send character to printer(or file) */
-	Char = STMemory_ReadWord(Params+SIZE_WORD);
-	Printer_TransferByteTo(Char);
+	c = STMemory_ReadWord(Params+SIZE_WORD);
+	Printer_TransferByteTo(c);
 	Regs[REG_D0] = -1;                /* Printer OK */
 
 	return TRUE;
@@ -974,7 +1055,7 @@ static BOOL GemDOS_DFree(Uint32 Params)
 */
 static BOOL GemDOS_MkDir(Uint32 Params)
 {
-	char szDirPath[MAX_PATH];
+	char szDirPath[MAX_GEMDOS_PATH];
 	char *pDirName;
 	int Drive;
 
@@ -1006,7 +1087,7 @@ static BOOL GemDOS_MkDir(Uint32 Params)
 */
 static BOOL GemDOS_RmDir(Uint32 Params)
 {
-	char szDirPath[MAX_PATH];
+	char szDirPath[MAX_GEMDOS_PATH];
 	char *pDirName;
 	int Drive;
 
@@ -1029,6 +1110,7 @@ static BOOL GemDOS_RmDir(Uint32 Params)
 	return FALSE;
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS ChDir
@@ -1036,38 +1118,47 @@ static BOOL GemDOS_RmDir(Uint32 Params)
 */
 static BOOL GemDOS_ChDir(Uint32 Params)
 {
-	char szDirPath[MAX_PATH];
 	char *pDirName;
 	int Drive;
 
 	/* Find new directory */
 	pDirName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+SIZE_WORD));
-#if FILE_DEBUG
 
-	fprintf(stderr,"chdir %s\n",pDirName);
+#if GEMDOS_FILE_DEBUG
+	Log_Printf(LOG_DEBUG, "Dsetpath(\"%s\")\n", pDirName);
 #endif
 
 	Drive = GemDOS_IsFileNameAHardDrive(pDirName);
 
 	if (ISHARDDRIVE(Drive))
 	{
-
 		struct stat buf;
+		char *psTempDirPath;
 
-		GemDOS_CreateHardDriveFileName(Drive,pDirName,szDirPath);
-		if (stat(szDirPath,&buf))
-		{ /* error */
+		/* Allocate temporary memory for path name: */
+		psTempDirPath = malloc(FILENAME_MAX);
+		if (!psTempDirPath)
+		{
+			perror("GemDOS_ChDir");
+			Regs[REG_D0] = GEMDOS_ENSMEM;
+			return TRUE;
+		}
+
+		GemDOS_CreateHardDriveFileName(Drive, pDirName, psTempDirPath);
+		if (stat(psTempDirPath, &buf))
+		{
+			/* error */
+			free(psTempDirPath);
 			Regs[REG_D0] = GEMDOS_EPTHNF;
 			return TRUE;
 		}
 
-		strcat(szDirPath, "/");
+		File_AddSlashToEndFileName(psTempDirPath);
+		File_MakeAbsoluteName(psTempDirPath);
 
-		/* remove any trailing slashes */
-		if (szDirPath[strlen(szDirPath)-2]=='/')
-			szDirPath[strlen(szDirPath)-1] = '\0';     /* then remove it! */
+		strcpy(emudrives[0]->fs_currpath, psTempDirPath);
+		free(psTempDirPath);
 
-		strcpy(emudrives[0]->fs_currpath, szDirPath);
 		Regs[REG_D0] = GEMDOS_EOK;
 		return TRUE;
 	}
@@ -1075,6 +1166,7 @@ static BOOL GemDOS_ChDir(Uint32 Params)
 	return FALSE;
 }
 
+
 /*-----------------------------------------------------------------------*/
 /*
   GEMDOS Create file
@@ -1082,7 +1174,7 @@ static BOOL GemDOS_ChDir(Uint32 Params)
 */
 static BOOL GemDOS_Create(Uint32 Params)
 {
-	char szActualFileName[MAX_PATH];
+	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	const char *rwflags[] =
 		{
@@ -1144,7 +1236,7 @@ static BOOL GemDOS_Create(Uint32 Params)
 */
 static BOOL GemDOS_Open(Uint32 Params)
 {
-	char szActualFileName[MAX_PATH];
+	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	const char *open_modes[] =
 		{ "rb", "wb", "r+", "rb"
@@ -1173,7 +1265,8 @@ static BOOL GemDOS_Open(Uint32 Params)
 		/* Open file */
 		FileHandles[Index].FileHandle =  fopen(szActualFileName, open_modes[Mode&0x03]);
 
-		sprintf(FileHandles[Index].szActualName,"%s",szActualFileName);
+		snprintf(FileHandles[Index].szActualName, sizeof(FileHandles[Index].szActualName),
+		         "%s", szActualFileName);
 
 		if (FileHandles[Index].FileHandle != NULL)
 		{
@@ -1296,14 +1389,15 @@ static BOOL GemDOS_Write(Uint32 Params)
 	if (!GemDOS_IsInvalidFileHandle(Handle))
 	{
 		nBytesWritten = fwrite(pBuffer, 1, Size, FileHandles[Handle].FileHandle);
-		if (nBytesWritten >= 0)
+		if (ferror(FileHandles[Handle].FileHandle))
+		{
+			Regs[REG_D0] = GEMDOS_EACCDN;      /* Access denied(ie read-only) */
+		}
+		else
 		{
 
 			Regs[REG_D0] = nBytesWritten;      /* OK */
 		}
-		else
-			Regs[REG_D0] = GEMDOS_EACCDN;      /* Access denied(ie read-only) */
-
 		return TRUE;
 	}
 #endif
@@ -1319,7 +1413,7 @@ static BOOL GemDOS_Write(Uint32 Params)
 static BOOL GemDOS_UnLink(Uint32 Params)
 {
 #ifdef ENABLE_SAVING
-	char szActualFileName[MAX_PATH];
+	char szActualFileName[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	int Drive;
 
@@ -1382,7 +1476,7 @@ static BOOL GemDOS_LSeek(Uint32 Params)
 */
 static BOOL GemDOS_Fattrib(Uint32 Params)
 {
-	char sActualFileName[MAX_PATH];
+	char sActualFileName[MAX_GEMDOS_PATH];
 	char *psFileName;
 	int nDrive;
 	int nRwFlag, nAttrib;
@@ -1395,7 +1489,7 @@ static BOOL GemDOS_Fattrib(Uint32 Params)
 	nAttrib = STMemory_ReadWord(Params+SIZE_WORD+SIZE_LONG+SIZE_WORD);
 
 #ifdef GEMDOS_VERBOSE
-	fprintf(stderr,"Fattrib('%s', %d, 0x%x)\n",psFileName, nRwFlag, nAttrib);
+	Log_Printf(LOG_DEBUG, "Fattrib('%s', %d, 0x%x)\n", psFileName, nRwFlag, nAttrib);
 #endif
 
 	if (ISHARDDRIVE(nDrive))
@@ -1444,7 +1538,7 @@ static int GemDOS_GetDir(Uint32 Params)
 	/* is it our drive? */
 	if ((Drive == 0 && CurrentDrive >= 2) || Drive >= 3)
 	{
-		char path[MAX_PATH];
+		char path[MAX_GEMDOS_PATH];
 		int i,len,c;
 
 		Regs[REG_D0] = GEMDOS_EOK;          /* OK */
@@ -1457,8 +1551,8 @@ static int GemDOS_GetDir(Uint32 Params)
 			c = path[i];
 			STMemory_WriteByte(Address+i, (c=='/' ? '\\' : c) );
 		}
-#if FILE_DEBUG
-		fprintf(stderr,"curdir %d -> %s\n",Drive,path);
+#if GEMDOS_FILE_DEBUG
+		Log_Printf(LOG_DEBUG, "GemDOS_GetDir (%d) = %s\n", Drive, path);
 #endif
 
 		return TRUE;
@@ -1476,7 +1570,10 @@ static int GemDOS_Pexec_LoadAndGo(Uint32 Params)
 {
 	/* add multiple disk support here too */
 	/* Hard-drive? */
-	if (CurrentDrive == emudrives[0]->hd_letter)
+	char *pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+2*SIZE_WORD));
+	int Drive = GemDOS_IsFileNameAHardDrive(pszFileName);
+
+	if (ISHARDDRIVE(Drive))
 	{
 		/* If not using A: or B:, use my own routines to load */
 		return CALL_PEXEC_ROUTINE;
@@ -1491,7 +1588,9 @@ static int GemDOS_Pexec_LoadAndGo(Uint32 Params)
 static int GemDOS_Pexec_LoadDontGo(Uint32 Params)
 {
 	/* Hard-drive? */
-	if (CurrentDrive == emudrives[0]->hd_letter)
+	char *pszFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+2*SIZE_WORD));
+	int Drive = GemDOS_IsFileNameAHardDrive(pszFileName);
+	if (ISHARDDRIVE(Drive))
 	{
 		return CALL_PEXEC_ROUTINE;
 	}
@@ -1506,10 +1605,16 @@ static int GemDOS_Pexec_LoadDontGo(Uint32 Params)
 static int GemDOS_Pexec(Uint32 Params)
 {
 	Uint16 Mode;
+	/*char *psFileName, *psCmdLine;*/
 
 	/* Find PExec mode */
 	Mode = STMemory_ReadWord(Params+SIZE_WORD);
 
+	/*
+	psFileName = (char *)STRAM_ADDR(STMemory_ReadLong(Params+2*SIZE_WORD));
+	psCmdLine = (char *)STRAM_ADDR(STMemory_ReadLong(Params+2*SIZE_WORD+SIZE_LONG));
+	*/
+
 	/* Re-direct as needed */
 	switch(Mode)
 	{
@@ -1535,7 +1640,7 @@ static int GemDOS_Pexec(Uint32 Params)
   GEMDOS Search Next
   Call 0x4F
 */
-static BOOL GemDOS_SNext(Uint32 Params)
+static BOOL GemDOS_SNext(void)
 {
 	struct dirent **temp;
 	int Index;
@@ -1559,7 +1664,7 @@ static BOOL GemDOS_SNext(Uint32 Params)
 		temp = InternalDTAs[Index].found;
 		if (PopulateDTA(InternalDTAs[Index].path, temp[InternalDTAs[Index].centry++]) == FALSE)
 		{
-			fprintf(stderr,"\tError setting DTA.\n");
+			Log_Printf(LOG_ERROR, "\tError setting DTA.\n");
 			return TRUE;
 		}
 
@@ -1578,8 +1683,8 @@ static BOOL GemDOS_SNext(Uint32 Params)
 */
 static BOOL GemDOS_SFirst(Uint32 Params)
 {
-	char szActualFileName[MAX_PATH];
-	char tempstr[MAX_PATH];
+	char szActualFileName[MAX_GEMDOS_PATH];
+	char tempstr[MAX_GEMDOS_PATH];
 	char *pszFileName;
 	struct dirent **files;
 	Uint16 Attr;
@@ -1667,7 +1772,7 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 		}
 
 		/* Scan for first file (SNext uses no parameters) */
-		GemDOS_SNext(0);
+		GemDOS_SNext();
 		/* increment DTA index */
 		DTAIndex++;
 		DTAIndex&=(MAX_DTAS_FILES-1);
@@ -1686,7 +1791,7 @@ static BOOL GemDOS_SFirst(Uint32 Params)
 static BOOL GemDOS_Rename(Uint32 Params)
 {
 	char *pszNewFileName,*pszOldFileName;
-	char szNewActualFileName[MAX_PATH],szOldActualFileName[MAX_PATH];
+	char szNewActualFileName[MAX_GEMDOS_PATH],szOldActualFileName[MAX_GEMDOS_PATH];
 	int NewDrive, OldDrive;
 
 	/* Read details from stack */
@@ -1902,7 +2007,7 @@ void GemDOS_OpCode(void)
 			RunOld = FALSE;
 		break;
 	 case 0x4f:
-		if (GemDOS_SNext(Params))
+		if (GemDOS_SNext())
 			RunOld = FALSE;
 		break;
 	 case 0x56:
