@@ -26,24 +26,12 @@ enum
   MAX_INTERRUPTS
 };
 
-/* Event timer structure - keeps next timer to occur in structure so don't need to check all entries */
-typedef struct
-{
-  BOOL bUsed;                   /* Is interrupt active? */
-  int Cycles;
-  void *pFunction;
-} INTERRUPTHANDLER;
-
-extern void *pIntHandlerFunctions[];
 extern int nCyclesOver;
-extern int nFrameCyclesOver;
 
 extern void Int_Reset(void);
 extern void Int_MemorySnapShot_Capture(BOOL bSave);
-extern int Int_HandlerFunctionToID(void *pHandlerFunction);
+extern int Int_HandlerFunctionToID(void (*pHandlerFunction)(void));
 extern void *Int_IDToHandlerFunction(int ID);
-extern int Int_FindFrameCycles(void);
-extern void Int_SetNewInterrupt(void);
 extern void Int_AcknowledgeInterrupt(void);
 extern void Int_AddAbsoluteInterrupt(int CycleTime, int Handler);
 extern void Int_AddRelativeInterrupt(int CycleTime, int Handler);
