@@ -13,11 +13,12 @@ typedef struct
 {
 	Uint32 Address;              /* ST hardware address */
 	int SpanInBytes;             /* E.g. SIZE_BYTE, SIZE_WORD or SIZE_LONG */
-	void *ReadFunc;              /* Read function */
-	void *WriteFunc;             /* Write function */
+	void (*ReadFunc)(void);      /* Read function */
+	void (*WriteFunc)(void);     /* Write function */
 } INTERCEPT_ACCESS_FUNC;
 
-extern INTERCEPT_ACCESS_FUNC IoMemTable_ST[];
-extern INTERCEPT_ACCESS_FUNC IoMemTable_STE[];
+extern const INTERCEPT_ACCESS_FUNC IoMemTable_ST[];
+extern const INTERCEPT_ACCESS_FUNC IoMemTable_STE[];
+extern const INTERCEPT_ACCESS_FUNC IoMemTable_TT[];
 
 #endif
