@@ -16,7 +16,7 @@
   the current instruction takes 20 cycles we will be 16 cycles late - this is handled in
   the adjust functions.
 */
-char Int_rcsid[] = "Hatari $Id: int.c,v 1.9 2005/08/06 12:32:09 thothy Exp $";
+const char Int_rcsid[] = "Hatari $Id: int.c,v 1.14 2006/06/26 23:03:09 thothy Exp $";
 
 #include "main.h"
 #include "dmaSnd.h"
@@ -29,8 +29,11 @@ char Int_rcsid[] = "Hatari $Id: int.c,v 1.9 2005/08/06 12:32:09 thothy Exp $";
 #include "video.h"
 
 
+int nCyclesOver = 0;
+
+
 /* List of possible interrupt handlers to be store in 'PendingInterruptTable', used for 'MemorySnapShot' */
-void *pIntHandlerFunctions[] =
+static void (* const pIntHandlerFunctions[])(void) =
 {
   NULL,
   Video_InterruptHandler_VBL,
@@ -46,10 +49,16 @@ void *pIntHandlerFunctions[] =
   NULL
 };
 
-INTERRUPTHANDLER InterruptHandlers[MAX_INTERRUPTS];
-int nCyclesOver=0;
-int nFrameCyclesOver=0;
-int ActiveInterrupt=0;
+/* Event timer structure - keeps next timer to occur in structure so don't need to check all entries */
+typedef struct
+{
+  BOOL bUsed;                   /* Is interrupt active? */
+  int Cycles;
+  void (*pFunction)(void);
+} INTERRUPTHANDLER;
+
+static INTERRUPTHANDLER InterruptHandlers[MAX_INTERRUPTS];
+static int ActiveInterrupt=0;
 
 
 /*-----------------------------------------------------------------------*/
@@ -96,7 +105,6 @@ void Int_MemorySnapShot_Capture(BOOL bSave)
     }
   }
   MemorySnapShot_Store(&nCyclesOver,sizeof(nCyclesOver));
-  MemorySnapShot_Store(&nFrameCyclesOver,sizeof(nFrameCyclesOver));
 }
 
 
@@ -104,7 +112,7 @@ void Int_MemorySnapShot_Capture(BOOL bSave)
 /*
   Convert interrupt handler function pointer to ID, used for saving
 */
-int Int_HandlerFunctionToID(void *pHandlerFunction)
+int Int_HandlerFunctionToID(void (*pHandlerFunction)(void))
 {
   int i;
 
@@ -119,7 +127,7 @@ int Int_HandlerFunctionToID(void *pHandlerFunction)
   }
   
   /* Didn't find one! Oops */
-  return(0);
+  return 0;
 }
 
 
@@ -134,21 +142,11 @@ void *Int_IDToHandlerFunction(int ID)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  Return number of clock cycles into retrace
-*/
-int Int_FindFrameCycles(void)
-{
-  return( nFrameCyclesOver + (InterruptHandlers[ActiveInterrupt].Cycles-PendingInterruptCount) );
-}
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Find next interrupt to occur, and store to global variables for decrement in instruction decode loop
 */
-void Int_SetNewInterrupt(void)
+static void Int_SetNewInterrupt(void)
 {
   int LowestCycleCount=999999,LowestInterrupt=0;
   int i;
@@ -191,7 +189,6 @@ static void Int_UpdateInterrupt(void)
     if (InterruptHandlers[i].bUsed)
       InterruptHandlers[i].Cycles -= CycleSubtract;
   }
-  nFrameCyclesOver += CycleSubtract;
 }
 
 
