@@ -8,7 +8,7 @@
 
   NOTE: The ST uses the joystick port 1 as the default controller.
 */
-char Joy_rcsid[] = "Hatari $Id: joy.c,v 1.8 2005/10/05 14:14:35 thothy Exp $";
+const char Joy_rcsid[] = "Hatari $Id: joy.c,v 1.13 2006/08/09 08:10:56 eerot Exp $";
 
 #include <SDL.h>
 
@@ -24,18 +24,19 @@ char Joy_rcsid[] = "Hatari $Id: joy.c,v 1.8 2005/10/05 14:14:35 thothy Exp $";
 #define JOY_BUTTON2  2
 
 
-SDL_Joystick *sdlJoystick[6] =          /* SDL's joystick structures */
+static SDL_Joystick *sdlJoystick[6] =   /* SDL's joystick structures */
 {
 	NULL, NULL, NULL, NULL, NULL, NULL
 };
-BOOL bJoystickWorking[6] =              /* Is joystick plugged in and working? */
+
+static BOOL bJoystickWorking[6] =       /* Is joystick plugged in and working? */
 {
 	FALSE, FALSE, FALSE, FALSE, FALSE, FALSE
 }; 
 
 int JoystickSpaceBar = FALSE;           /* State of space-bar on joystick button 2 */
 static Uint8 nJoyKeyEmu[6];
-Uint16 nSteJoySelect;
+static Uint16 nSteJoySelect;
 
 
 /*-----------------------------------------------------------------------*/
@@ -80,6 +81,26 @@ void Joy_Init(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  Close the (real) joysticks.
+*/
+void Joy_UnInit(void)
+{
+	int i, nPadsConnected;
+
+	nPadsConnected = SDL_NumJoysticks();
+
+	for (i = 0; i < nPadsConnected && i < 6; i++)
+	{
+		if (bJoystickWorking[i] == TRUE)
+		{
+			SDL_JoystickClose(sdlJoystick[i]);
+		}
+	}
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Read details from joystick using SDL calls
@@ -216,17 +237,49 @@ static int Joy_GetFireButtons(int nStJoyId)
 
 /*-----------------------------------------------------------------------*/
 /*
-  Toggle cursor emulation
+  Set joystick cursor emulation for given port.  This assumes that
+  if the same keys have been defined for "cursor key emulation" in
+  other ports, the emulation for them has been switched off. Returns
+  1 if the port number was OK, zero for error.
+*/
+BOOL Joy_SetCursorEmulation(int port)
+{
+	if (port < 0 || port >= JOYSTICK_COUNT) {
+		return 0;
+	}
+	ConfigureParams.Joysticks.Joy[port].nJoystickMode = JOYSTICK_KEYBOARD;
+	return 1;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Toggle joystick cursor emulation between port 0, port 1 and being off
+  from them. When it's turned off from them, the port's previous state
+  is restored
 */
 void Joy_ToggleCursorEmulation(void)
 {
-	/* Toggle joystick 1 keyboard emulation */
-	if (ConfigureParams.Joysticks.Joy[1].nJoystickMode != JOYSTICK_DISABLED)
-	{
-		if (ConfigureParams.Joysticks.Joy[1].nJoystickMode == JOYSTICK_REALSTICK)
-			ConfigureParams.Joysticks.Joy[1].nJoystickMode = JOYSTICK_KEYBOARD;
-		else
-			ConfigureParams.Joysticks.Joy[1].nJoystickMode = JOYSTICK_REALSTICK;
+	static int saved[2] = { JOYSTICK_DISABLED, JOYSTICK_DISABLED };
+	int i, state, port = 2;
+	for (i = 0; i < 2; i++) {
+		state = ConfigureParams.Joysticks.Joy[i].nJoystickMode;
+		if (state == JOYSTICK_KEYBOARD) {
+			port = i;
+		} else {
+			saved[i] = state;
+		}
+	}
+	switch (port) {
+	case 0:  /* (only) in port 0, disable cursor emu */
+		ConfigureParams.Joysticks.Joy[0].nJoystickMode = saved[0];
+		break;
+	case 1:  /* (at least) in port 1, switch cursor emu to port 0 */
+		ConfigureParams.Joysticks.Joy[1].nJoystickMode = saved[1];
+		ConfigureParams.Joysticks.Joy[0].nJoystickMode = JOYSTICK_KEYBOARD;
+		break;
+	default:  /* neither in port 0 or 1, enable cursor emu to port 1 */
+		ConfigureParams.Joysticks.Joy[1].nJoystickMode = JOYSTICK_KEYBOARD;
 	}
 }
 
@@ -240,7 +293,7 @@ BOOL Joy_KeyDown(int symkey, int modkey)
 {
 	int i;
 
-	for (i = 0; i < 6; i++)
+	for (i = 0; i < JOYSTICK_COUNT; i++)
 	{
 		if (ConfigureParams.Joysticks.Joy[i].nJoystickMode == JOYSTICK_KEYBOARD
 		    && !(modkey & KMOD_SHIFT))
@@ -286,7 +339,7 @@ BOOL Joy_KeyUp(int symkey, int modkey)
 {
 	int i;
 
-	for (i = 0; i < 6; i++)
+	for (i = 0; i < JOYSTICK_COUNT; i++)
 	{
 		if (ConfigureParams.Joysticks.Joy[i].nJoystickMode == JOYSTICK_KEYBOARD
 		    && !(modkey & KMOD_SHIFT))
