@@ -6,7 +6,7 @@
 
   Here we process a key press and the remapping of the scancodes.
 */
-char Keymap_rcsid[] = "Hatari $Id: keymap.c,v 1.22 2005/09/26 18:24:09 thothy Exp $";
+const char Keymap_rcsid[] = "Hatari $Id: keymap.c,v 1.28 2006/08/09 08:14:24 eerot Exp $";
 
 #include "main.h"
 #include "keymap.h"
@@ -17,6 +17,7 @@ char Keymap_rcsid[] = "Hatari $Id: keymap.c,v 1.22 2005/09/26 18:24:09 thothy Ex
 #include "shortcut.h"
 #include "screen.h"
 #include "debugui.h"
+#include "log.h"
 
 
 /*-----------------------------------------------------------------------*/
@@ -94,10 +95,6 @@ Ins   Del
 */
 
 
-/* Mask of valid shortcut key modifiers */
-#define KEYMAP_SHORTCUTMODS  (KMOD_RALT|KMOD_LMETA|KMOD_RMETA|KMOD_MODE)
-
-
 /* SDL symbolic key to ST scan code mapping table */
 static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
 {
@@ -321,7 +318,7 @@ static Uint8 SdlSymToSdlScan[SDLK_LAST];
 
 
 /* List of ST scan codes to NOT de-bounce when running in maximum speed */
-static char DebounceExtendedKeys[] =
+static const char DebounceExtendedKeys[] =
 {
   0x1d,  /* CTRL */
   0x2a,  /* Left SHIFT */
@@ -595,7 +592,7 @@ char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 void Keymap_LoadRemapFile(char *pszFileName)
 {
   char szString[1024];
-  unsigned int STScanCode, PCKeyCode;
+  int STScanCode, PCKeyCode;
   FILE *in;
 
   /* Initialize table with default values */
@@ -622,13 +619,20 @@ void Keymap_LoadRemapFile(char *pszFileName)
           /* Read values */
           sscanf(szString, "%d,%d", &PCKeyCode, &STScanCode);
           /* Store into remap table, check both value within range */
-          if ( (PCKeyCode>=0) && (PCKeyCode<SDLK_LAST) && (STScanCode>=0) && (STScanCode<256) )
+          if ( (PCKeyCode>=0) && (PCKeyCode<SDLK_LAST) &&
+	       (STScanCode>=0) && (STScanCode<256) )
             LoadedKeyToSTScanCode[PCKeyCode] = STScanCode;
         }
       }
 
       fclose(in);
     }
+    else
+    {
+      Log_Printf(LOG_DEBUG,
+		 "Keymap_LoadRemapFile: failed to open keymap file '%s'\n",
+		 pszFileName);
+    }
   }
 }
 
@@ -669,7 +673,7 @@ static BOOL Keymap_DebounceSTKey(char STScanCode)
 */
 void Keymap_DebounceAllKeys(void)
 {
-  unsigned int key;
+  SDLKey key;
   char STScanCode;
   SDL_keysym tmpKeySym;
 
@@ -680,10 +684,10 @@ void Keymap_DebounceAllKeys(void)
      return;
   }
 
-  tmpKeySym.mod = 0;
+  tmpKeySym.mod = KMOD_NONE;
 
   /* Now run through each PC key looking for ones held down */
-  for(key = 0; key < SDLK_LAST; key++)
+  for(key = SDLK_FIRST; key < SDLK_LAST; key++)
   {
     /* Is key held? */
     if(Keyboard.KeyStates[key])
@@ -718,16 +722,8 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
 
   /*fprintf(stderr, "keydown: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
 
-  /* If pressed short-cut key, retain keypress until safe to execute (start of VBL) */
-  if ((modkey & KEYMAP_SHORTCUTMODS) || symkey == SDLK_F11 || symkey == SDLK_F12)
-  {
-    ShortCutKey.Key = symkey;
-    if (modkey & KMOD_CTRL)
-      ShortCutKey.bCtrlPressed = TRUE;
-    if (modkey & KMOD_SHIFT)
-      ShortCutKey.bShiftPressed = TRUE;
+  if (ShortCut_CheckKeys(modkey, symkey, 1))
     return;
-  }
 
   /* If using joystick emulation via keyboard, DON'T send keys to keyboard processor!!!
    * Some games use keyboard as pause! */
@@ -775,10 +771,9 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
   /*fprintf(stderr, "keyup: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
 
   /* Ignore short-cut keys here */
-  if ((modkey & KEYMAP_SHORTCUTMODS) || symkey == SDLK_F11 || symkey == SDLK_F12)
-  {
+
+  if (ShortCut_CheckKeys(modkey, symkey, 0))
     return;
-  }
 
   /* If using keyboard emulation, DON'T send keys to keyboard processor!!!
    * Some games use keyboard as pause! */
