@@ -14,7 +14,7 @@
   It shows the main details of the chip's behaviour with regard to interrupts
   and pending/service bits.
 */
-char MFP_rcsid[] = "Hatari $Id: mfp.c,v 1.21 2005/10/04 12:43:39 thothy Exp $";
+const char MFP_rcsid[] = "Hatari $Id: mfp.c,v 1.26 2006/02/08 22:49:27 eerot Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -59,19 +59,20 @@ Input -----/             |         ------------------------              |
 
 /* MFP Registers */
 Uint8 MFP_GPIP;                     /* General Purpose Pins */
-Uint8 MFP_AER,MFP_DDR;              /* Active Edge Register, Data Direction Register */
 Uint8 MFP_IERA,MFP_IERB;            /* Interrupt Enable Registers A,B  0xfffa07,0xfffa09 */
 Uint8 MFP_IPRA,MFP_IPRB;            /* Interrupt Pending Registers A,B  0xfffa0b,0xfffa0d */
-Uint8 MFP_ISRA,MFP_ISRB;            /* Interrupt In-Service Registers A,B  0xfffa0f,0xfffa11 */
-Uint8 MFP_IMRA,MFP_IMRB;            /* Interrupt Mask Registers A,B  0xfffa13,0xfffa15 */
 Uint8 MFP_VR;                       /* Vector Register  0xfffa17 */
 Uint8 MFP_TACR,MFP_TBCR,MFP_TCDCR;  /* Timer A,B,C+D Control Registers */
-Uint8 MFP_TADR,MFP_TBDR;            /* Timer A,B Data Registers */
-Uint8 MFP_TCDR,MFP_TDDR;            /* Timer C,D Data Registers */
-Uint8 MFP_TA_MAINCOUNTER;           /* Timer A Main Counter (internal to MFP) */
-Uint8 MFP_TB_MAINCOUNTER;           /* Timer B Main Counter */
-Uint8 MFP_TC_MAINCOUNTER;           /* Timer C Main Counter (these are temp's, set when read as) */
-Uint8 MFP_TD_MAINCOUNTER;           /* Timer D Main Counter (as done via interrupts) */
+
+static Uint8 MFP_AER,MFP_DDR;       /* Active Edge Register, Data Direction Register */
+static Uint8 MFP_ISRA,MFP_ISRB;     /* Interrupt In-Service Registers A,B  0xfffa0f,0xfffa11 */
+static Uint8 MFP_IMRA,MFP_IMRB;     /* Interrupt Mask Registers A,B  0xfffa13,0xfffa15 */
+static Uint8 MFP_TADR,MFP_TBDR;     /* Timer A,B Data Registers */
+static Uint8 MFP_TCDR,MFP_TDDR;     /* Timer C,D Data Registers */
+static Uint8 MFP_TA_MAINCOUNTER;    /* Timer A Main Counter (internal to MFP) */
+static Uint8 MFP_TB_MAINCOUNTER;    /* Timer B Main Counter */
+static Uint8 MFP_TC_MAINCOUNTER;    /* Timer C Main Counter (these are temp's, set when read as) */
+static Uint8 MFP_TD_MAINCOUNTER;    /* Timer D Main Counter (as done via interrupts) */
 
 /* CPU clock cycle counts for each timer */
 static int TimerAClockCycles=0;
@@ -89,7 +90,7 @@ static int nTimerDFakeValue;        /* Faked Timer-D data register for the Timer
  Now, Timer C set on a delay of 192($C0) and a preset DIV of 64 is 200Hz
  This makes the table entry 208.66666*192=40064(200Hz)
 */
-static float MFPTimerToCPUCycleTable[] = {
+static const float MFPTimerToCPUCycleTable[] = {
    0,             /* Timer Stop */
    13.04166667f,  /* Div by 4  */
    32.60416667f,  /* Div by 10 */
@@ -184,6 +185,26 @@ static void MFP_Exception(int Interrupt)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  This is called whenever the MFP_IPRA or MFP_IPRB registers are modified.
+  We set the special flag SPCFLAG_MFP accordingly (to say if an MFP interrupt
+  is to be checked) so we only have one compare during the decode
+  instruction loop.
+*/
+static void MFP_UpdateFlags(void)
+{
+  if( MFP_IPRA|MFP_IPRB )
+  {
+    set_special(SPCFLAG_MFP);
+  }
+  else
+  {
+    unset_special(SPCFLAG_MFP);
+  }
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Test interrupt request to see if can cause exception,return TRUE if pass vector
@@ -264,26 +285,6 @@ void MFP_CheckPendingInterrupts(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  This is called whenever the MFP_IPRA or MFP_IPRB registers are modified.
-  We set the special flag SPCFLAG_MFP accordingly (to say if an MFP interrupt
-  is to be checked) so we only have one compare during the decode
-  instruction loop.
-*/
-void MFP_UpdateFlags(void)
-{
-  if( MFP_IPRA|MFP_IPRB )
-  {
-    set_special(SPCFLAG_MFP);
-  }
-  else
-  {
-    unset_special(SPCFLAG_MFP);
-  }
-}
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Interrupt Channel is active, set pending bit so can be serviced
@@ -447,7 +448,7 @@ static Uint8 MFP_ReadTimerCD(Uint8 TimerControl, Uint8 TimerData, Uint8 MainCoun
   Start Timer A
   (This does not start the EventCount mode time as this is taken care of by the HBL)
 */
-void MFP_StartTimerA(void)
+static void MFP_StartTimerA(void)
 {
   TimerAClockCycles = MFP_StartTimer_AB(MFP_TACR,MFP_TADR,INTERRUPT_MFP_TIMERA,TRUE);
 }
@@ -457,7 +458,7 @@ void MFP_StartTimerA(void)
 /*
   Read Timer A
 */
-void MFP_ReadTimerA(void)
+static void MFP_ReadTimerA(void)
 {
   MFP_TA_MAINCOUNTER = MFP_ReadTimer_AB(MFP_TACR,MFP_TA_MAINCOUNTER,TimerAClockCycles,INTERRUPT_MFP_TIMERA);
 }
@@ -468,7 +469,7 @@ void MFP_ReadTimerA(void)
   Start Timer B
   (This does not start the EventCount mode time as this is taken care of by the HBL)
 */
-void MFP_StartTimerB(void)
+static void MFP_StartTimerB(void)
 {
   TimerBClockCycles = MFP_StartTimer_AB(MFP_TBCR,MFP_TBDR,INTERRUPT_MFP_TIMERB,TRUE);
 }
@@ -478,7 +479,7 @@ void MFP_StartTimerB(void)
 /*
   Read Timer B
 */
-void MFP_ReadTimerB(void)
+static void MFP_ReadTimerB(void)
 {
   MFP_TB_MAINCOUNTER = MFP_ReadTimer_AB(MFP_TBCR,MFP_TB_MAINCOUNTER,TimerBClockCycles,INTERRUPT_MFP_TIMERB);
 }
@@ -488,7 +489,7 @@ void MFP_ReadTimerB(void)
 /*
   Start Timer C
 */
-void MFP_StartTimerC(void)
+static void MFP_StartTimerC(void)
 {
   TimerCClockCycles = MFP_StartTimer_CD(MFP_TCDCR>>4,MFP_TCDR,INTERRUPT_MFP_TIMERC,TRUE);
 }
@@ -498,7 +499,7 @@ void MFP_StartTimerC(void)
 /*
   Read Timer C
 */
-void MFP_ReadTimerC(void)
+static void MFP_ReadTimerC(void)
 {
   MFP_TC_MAINCOUNTER = MFP_ReadTimerCD(MFP_TCDCR>>4,MFP_TCDR,MFP_TC_MAINCOUNTER,TimerCClockCycles,INTERRUPT_MFP_TIMERC);
 }
@@ -508,7 +509,7 @@ void MFP_ReadTimerC(void)
 /*
   Start Timer D
 */
-void MFP_StartTimerD(void)
+static void MFP_StartTimerD(void)
 {
   TimerDClockCycles = MFP_StartTimer_CD(MFP_TCDCR,MFP_TDDR,INTERRUPT_MFP_TIMERD,TRUE);
 }
@@ -518,7 +519,7 @@ void MFP_StartTimerD(void)
 /*
   Read Timer D
 */
-void MFP_ReadTimerD(void)
+static void MFP_ReadTimerD(void)
 {
   MFP_TD_MAINCOUNTER = MFP_ReadTimerCD(MFP_TCDCR,MFP_TDDR,MFP_TC_MAINCOUNTER,TimerDClockCycles,INTERRUPT_MFP_TIMERD);
 }
@@ -615,6 +616,8 @@ void MFP_InterruptHandler_TimerD(void)
 */
 void MFP_GPIP_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	if (!bUseHighRes)
 		MFP_GPIP |= 0x80;       /* Color monitor -> set top bit */
 	else
@@ -640,6 +643,8 @@ void MFP_GPIP_ReadByte(void)
 		}
 	}
 
+	FDC_GpipRead();
+
 	IoMem[0xfffa01] = MFP_GPIP;
 }
 
@@ -649,6 +654,8 @@ void MFP_GPIP_ReadByte(void)
 */
 void MFP_ActiveEdge_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa03] = MFP_AER;
 }
 
@@ -658,6 +665,8 @@ void MFP_ActiveEdge_ReadByte(void)
 */
 void MFP_DataDirection_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa05] = MFP_DDR;
 }
 
@@ -667,6 +676,8 @@ void MFP_DataDirection_ReadByte(void)
 */
 void MFP_EnableA_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa07] = MFP_IERA;
 }
 
@@ -676,6 +687,8 @@ void MFP_EnableA_ReadByte(void)
 */
 void MFP_EnableB_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa09] = MFP_IERB;
 }
 
@@ -685,6 +698,8 @@ void MFP_EnableB_ReadByte(void)
 */
 void MFP_PendingA_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa0b] = MFP_IPRA;
 }
 
@@ -694,6 +709,8 @@ void MFP_PendingA_ReadByte(void)
 */
 void MFP_PendingB_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa0d] = MFP_IPRB;
 }
 
@@ -703,6 +720,8 @@ void MFP_PendingB_ReadByte(void)
 */
 void MFP_InServiceA_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa0f] = MFP_ISRA;
 }
 
@@ -712,6 +731,8 @@ void MFP_InServiceA_ReadByte(void)
 */
 void MFP_InServiceB_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa11] = MFP_ISRB;
 }
 
@@ -721,6 +742,8 @@ void MFP_InServiceB_ReadByte(void)
 */
 void MFP_MaskA_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa13] = MFP_IMRA;
 }
 
@@ -730,6 +753,8 @@ void MFP_MaskA_ReadByte(void)
 */
 void MFP_MaskB_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa15] = MFP_IMRB;
 }
 
@@ -739,6 +764,8 @@ void MFP_MaskB_ReadByte(void)
 */
 void MFP_VectorReg_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa17] = MFP_VR;
 }
 
@@ -748,6 +775,8 @@ void MFP_VectorReg_ReadByte(void)
 */
 void MFP_TimerACtrl_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa19] = MFP_TACR;
 }
 
@@ -757,6 +786,8 @@ void MFP_TimerACtrl_ReadByte(void)
 */
 void MFP_TimerBCtrl_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa1b] = MFP_TBCR;
 }
 
@@ -766,6 +797,8 @@ void MFP_TimerBCtrl_ReadByte(void)
 */
 void MFP_TimerCDCtrl_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	IoMem[0xfffa1d] = MFP_TCDCR;
 }
 
@@ -775,6 +808,8 @@ void MFP_TimerCDCtrl_ReadByte(void)
 */
 void MFP_TimerAData_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	if (MFP_TACR != 8)          /* Is event count? Need to re-calculate counter */
 		MFP_ReadTimerA();       /* Stores result in 'MFP_TA_MAINCOUNTER' */
 
@@ -787,6 +822,8 @@ void MFP_TimerAData_ReadByte(void)
 */
 void MFP_TimerBData_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	if (MFP_TBCR != 8)          /* Is event count? Need to re-calculate counter */
 		MFP_ReadTimerB();       /* Stores result in 'MFP_TB_MAINCOUNTER' */
 
@@ -799,6 +836,8 @@ void MFP_TimerBData_ReadByte(void)
 */
 void MFP_TimerCData_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_ReadTimerC();        /* Stores result in 'MFP_TC_MAINCOUNTER' */
 
 	IoMem[0xfffa23] = MFP_TC_MAINCOUNTER;
@@ -812,6 +851,8 @@ void MFP_TimerDData_ReadByte(void)
 {
 	Uint32 pc = m68k_getpc();
 
+	M68000_WaitState(4);
+
 	if (ConfigureParams.System.bPatchTimerD && pc >= TosAddress && pc <= TosAddress + TosSize)
 	{
 		/* Trick the tos to believe it was changed: */
@@ -831,6 +872,8 @@ void MFP_TimerDData_ReadByte(void)
 */
 void MFP_GPIP_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	/* Nothing... */
 	/*fprintf(stderr, "Write to GPIP: %x\n", (int)IoMem[0xfffa01]);*/
 	/*MFP_GPIP = IoMem[0xfffa01];*/   /* TODO: What are the GPIP pins good for? */
@@ -842,6 +885,8 @@ void MFP_GPIP_WriteByte(void)
 */
 void MFP_ActiveEdge_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_AER = IoMem[0xfffa03];
 }
 
@@ -851,6 +896,8 @@ void MFP_ActiveEdge_WriteByte(void)
 */
 void MFP_DataDirection_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_DDR = IoMem[0xfffa05];
 }
 
@@ -860,6 +907,8 @@ void MFP_DataDirection_WriteByte(void)
 */
 void MFP_EnableA_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IERA = IoMem[0xfffa07];
 	MFP_IPRA &= MFP_IERA;
 	MFP_UpdateFlags();
@@ -874,6 +923,8 @@ void MFP_EnableA_WriteByte(void)
 */
 void MFP_EnableB_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IERB = IoMem[0xfffa09];
 	MFP_IPRB &= MFP_IERB;
 	MFP_UpdateFlags();
@@ -888,6 +939,8 @@ void MFP_EnableB_WriteByte(void)
 */
 void MFP_PendingA_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IPRA &= IoMem[0xfffa0b];        /* Cannot set pending bits - only clear via software */
 	MFP_UpdateFlags();                  /* Check if any interrupts pending */
 }
@@ -898,6 +951,8 @@ void MFP_PendingA_WriteByte(void)
 */
 void MFP_PendingB_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IPRB &= IoMem[0xfffa0d];
 	MFP_UpdateFlags();                  /* Check if any interrupts pending */
 }
@@ -908,6 +963,8 @@ void MFP_PendingB_WriteByte(void)
 */
 void MFP_InServiceA_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_ISRA &= IoMem[0xfffa0f];        /* Cannot set in-service bits - only clear via software */
 }
 
@@ -917,6 +974,8 @@ void MFP_InServiceA_WriteByte(void)
 */
 void MFP_InServiceB_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_ISRB &= IoMem[0xfffa11];        /* Cannot set in-service bits - only clear via software */
 }
 
@@ -926,6 +985,8 @@ void MFP_InServiceB_WriteByte(void)
 */
 void MFP_MaskA_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IMRA = IoMem[0xfffa13];
 }
 
@@ -935,6 +996,8 @@ void MFP_MaskA_WriteByte(void)
 */
 void MFP_MaskB_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_IMRB = IoMem[0xfffa15];
 }
 
@@ -945,12 +1008,18 @@ void MFP_MaskB_WriteByte(void)
 void MFP_VectorReg_WriteByte(void)
 {
 	Uint8 old_vr;
+
+	M68000_WaitState(4);
+
 	old_vr = MFP_VR;                    /* Copy for checking if set mode */
 	MFP_VR = IoMem[0xfffa17];
+
 	if ((MFP_VR^old_vr) & 0x08)         /* Test change in end-of-interrupt mode */
 	{
-		if (MFP_VR & 0x08)              /* Mode did change but was it to automatic mode? (ie bit is a zero) */
-		{                               /* We are now in automatic mode, so clear all in-service bits! */
+		/* Mode did change but was it to automatic mode? (ie bit is a zero) */
+		if (!(MFP_VR & 0x08))
+		{
+			/* We are now in automatic mode, so clear all in-service bits! */
 			MFP_ISRA = 0;
 			MFP_ISRB = 0;
 		}
@@ -964,8 +1033,12 @@ void MFP_VectorReg_WriteByte(void)
 void MFP_TimerACtrl_WriteByte(void)
 {
 	Uint8 old_tacr;
+
+	M68000_WaitState(4);
+
 	old_tacr = MFP_TACR;                /* Remember old control state */
 	MFP_TACR = IoMem[0xfffa19] & 0x0f;  /* Mask, Fish (auto160) writes into top nibble! */
+
 	if ((MFP_TACR^old_tacr) & 0x0f)     /* Check if Timer A control changed */
 		MFP_StartTimerA();              /* Reset timers if need to */
 }
@@ -977,8 +1050,12 @@ void MFP_TimerACtrl_WriteByte(void)
 void MFP_TimerBCtrl_WriteByte(void)
 {
 	Uint8 old_tbcr;
+
+	M68000_WaitState(4);
+
 	old_tbcr = MFP_TBCR;                /* Remember old control state */
 	MFP_TBCR = IoMem[0xfffa1b] & 0x0f;  /* Mask, Fish (auto160) writes into top nibble! */
+
 	if ((MFP_TBCR^old_tbcr) & 0x0f)     /* Check if Timer B control changed */
 		MFP_StartTimerB();              /* Reset timers if need to */
 }
@@ -991,6 +1068,8 @@ void MFP_TimerCDCtrl_WriteByte(void)
 {
 	Uint8 old_tcdcr;
 
+	M68000_WaitState(4);
+
 	old_tcdcr = MFP_TCDCR;              /* Remember old control state */
 	MFP_TCDCR = IoMem[0xfffa1d];        /* Store new one */
 
@@ -1033,7 +1112,10 @@ void MFP_TimerCDCtrl_WriteByte(void)
 */
 void MFP_TimerAData_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_TADR = IoMem[0xfffa1f];         /* Store into data register */
+
 	if (MFP_TACR == 0)                  /* Now check if timer is running - if so do not set */
 	{
 		MFP_TA_MAINCOUNTER = MFP_TADR;  /* Timer is off, store to main counter */
@@ -1047,7 +1129,10 @@ void MFP_TimerAData_WriteByte(void)
 */
 void MFP_TimerBData_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_TBDR = IoMem[0xfffa21];         /* Store into data register */
+
 	if (MFP_TBCR == 0)                  /* Now check if timer is running - if so do not set */
 	{
 		MFP_TB_MAINCOUNTER = MFP_TBDR;  /* Timer is off, store to main counter */
@@ -1061,7 +1146,10 @@ void MFP_TimerBData_WriteByte(void)
 */
 void MFP_TimerCData_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	MFP_TCDR = IoMem[0xfffa23];         /* Store into data register */
+
 	if ((MFP_TCDCR&0x70) == 0)          /* Now check if timer is running - if so do not set */
 	{
 		MFP_StartTimerC();              /* Add our interrupt */
@@ -1076,6 +1164,8 @@ void MFP_TimerDData_WriteByte(void)
 {
 	Uint32 pc = m68k_getpc();
 
+	M68000_WaitState(4);
+
 	/* Need to change baud rate of RS232 emulation? */
 	if (ConfigureParams.RS232.bEnableRS232 && (IoMem[0xfffa1d] & 0x07))
 	{
