@@ -6,7 +6,7 @@
 
   Reset emulation state.
 */
-char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.10 2005/08/21 21:13:43 thothy Exp $";
+const char Reset_rcsid[] = "Hatari $Id: reset.c,v 1.14 2006/02/21 14:15:35 thothy Exp $";
 
 #include "main.h"
 #include "cart.h"
@@ -60,13 +60,10 @@ int Reset_ST(BOOL bCold)
   {
     int ret;
 
-    STMemory_Clear(0x00000000, 0x00400000);   /* Clear First 4Mb */
     STMemory_Clear(0x00e00000, 0x00ffffff);   /* Clear Upper memory */
-    STMemory_WriteLong(4, 0x00fc0000);        /* Set reset vector */
-    STMemory_WriteLong(0, 0x0000f000);        /* And reset stack pointer */
 
     Floppy_GetBootDrive();      /* Find which device to boot from(A: or C:) */
-    Cart_LoadImage();           /* Load program into cartridge memory. Used for gemdos loading */
+    Cart_ResetImage();          /* Load cartridge program into ROM memory. */
 
     ret = TOS_LoadImage();      /* Load TOS, writes into cartridge memory */
     if(ret)
@@ -92,7 +89,7 @@ int Reset_ST(BOOL bCold)
   M68000_Reset(bCold);          /* Reset CPU */
 
   /* And VBL interrupt, MUST always be one interrupt ready to trigger */
-  Int_AddAbsoluteInterrupt(CYCLES_ENDLINE, INTERRUPT_VIDEO_ENDLINE);
+  Int_AddAbsoluteInterrupt(nCyclesPerLine, INTERRUPT_VIDEO_ENDLINE);
   Int_AddAbsoluteInterrupt(CYCLES_HBL, INTERRUPT_VIDEO_HBL);
   Int_AddAbsoluteInterrupt(CYCLES_PER_FRAME, INTERRUPT_VIDEO_VBL);
 
