@@ -13,7 +13,7 @@
   the bytes into an input buffer. This method fits in with the internet code
   which also reads data into a buffer.
 */
-char RS232_rcsid[] = "Hatari $Id: rs232.c,v 1.17 2005/09/26 15:20:14 thothy Exp $";
+const char RS232_rcsid[] = "Hatari $Id: rs232.c,v 1.22 2006/07/27 19:57:44 thothy Exp $";
 
 #ifndef HAVE_TERMIOS_H
 #define HAVE_TERMIOS_H 1
@@ -30,6 +30,7 @@ char RS232_rcsid[] = "Hatari $Id: rs232.c,v 1.17 2005/09/26 15:20:14 thothy Exp
 
 #include "main.h"
 #include "configuration.h"
+#include "m68000.h"
 #include "mfp.h"
 #include "stMemory.h"
 #include "rs232.h"
@@ -53,16 +54,17 @@ char RS232_rcsid[] = "Hatari $Id: rs232.c,v 1.17 2005/09/26 15:20:14 thothy Exp
 #endif
 
 
-BOOL bConnectedRS232 = FALSE;       /* Connection to RS232? */
-static FILE *hComIn = NULL;         /* Handle to file for reading */
-static FILE *hComOut = NULL;        /* Handle to file for writing */
-SDL_Thread *RS232Thread = NULL;     /* Thread handle for reading incoming data */
-unsigned char InputBuffer_RS232[MAX_RS232INPUT_BUFFER];
-int InputBuffer_Head=0, InputBuffer_Tail=0;
-SDL_sem* pSemFreeBuf;               /* Semaphore to sync free space in InputBuffer_RS232 */
+BOOL bConnectedRS232 = FALSE;      /* Connection to RS232? */
+static FILE *hComIn = NULL;        /* Handle to file for reading */
+static FILE *hComOut = NULL;       /* Handle to file for writing */
+
+static unsigned char InputBuffer_RS232[MAX_RS232INPUT_BUFFER];
+static int InputBuffer_Head=0, InputBuffer_Tail=0;
 
 
-#if HAVE_TERMIOS_H && !HAVE_CFMAKERAW
+#if HAVE_TERMIOS_H
+
+#if !HAVE_CFMAKERAW
 static inline void cfmakeraw(struct termios *termios_p)
 {
 	termios_p->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
@@ -73,6 +75,241 @@ static inline void cfmakeraw(struct termios *termios_p)
 }
 #endif
 
+/*-----------------------------------------------------------------------*/
+/*
+  Set serial line parameters to "raw" mode.
+*/
+static BOOL RS232_SetRawMode(FILE *fhndl)
+{
+	struct termios termmode;
+	int fd;
+
+	memset (&termmode, 0, sizeof(termmode));    /* Init with zeroes */
+	fd = fileno(fhndl);                         /* Get file descriptor */
+
+	if (isatty(fd))
+	{
+		if (tcgetattr(fd, &termmode) != 0)
+			return FALSE;
+
+		/* Set "raw" mode: */
+		termmode.c_cc[VMIN] = 1;
+		termmode.c_cc[VTIME] = 0;
+		cfmakeraw(&termmode);
+		if (tcsetattr(fd, TCSADRAIN, &termmode) != 0)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+/*-----------------------------------------------------------------------*/
+/*
+  Set hardware configuration of RS-232:
+  - Bits per character
+  - Parity
+  - Start/stop bits
+*/
+static BOOL RS232_SetBitsConfig(int fd, int nCharSize, int nStopBits, BOOL bUseParity, BOOL bEvenParity)
+{
+	struct termios termmode;
+
+	memset (&termmode, 0, sizeof(termmode));    /* Init with zeroes */
+
+	if (isatty(fd))
+	{
+		if (tcgetattr(fd, &termmode) != 0)
+		{
+			Dprintf(("RS232_SetBitsConfig: tcgetattr failed.\n"));
+			return FALSE;
+		}
+
+		/* Set the character size: */
+		termmode.c_cflag &= ~CSIZE;
+		switch (nCharSize)
+		{
+		 case 8: termmode.c_cflag |= CS8; break;
+		 case 7: termmode.c_cflag |= CS7; break;
+		 case 6: termmode.c_cflag |= CS6; break;
+		 case 5: termmode.c_cflag |= CS5; break;
+		}
+
+		/* Set stop bits: */
+		if (nStopBits >= 2)
+			termmode.c_oflag |= CSTOPB;
+		else
+			termmode.c_oflag &= ~CSTOPB;
+
+		/* Parity bit: */
+		if (bUseParity)
+			termmode.c_cflag |= PARENB;
+		else
+			termmode.c_cflag &= ~PARENB;
+
+		if (bEvenParity)
+			termmode.c_cflag &= ~PARODD;
+		else
+			termmode.c_cflag |= PARODD;
+
+		/* Now store the configuration: */
+		if (tcsetattr(fd, TCSADRAIN, &termmode) != 0)
+		{
+			Dprintf(("RS232_SetBitsConfig: tcsetattr failed.\n"));
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+#endif /* HAVE_TERMIOS_H */
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Open file on COM port.
+*/
+static BOOL RS232_OpenCOMPort(void)
+{
+	bConnectedRS232 = FALSE;
+
+	/* Create our COM file for output */
+	hComOut = fopen(ConfigureParams.RS232.szOutFileName, "wb"); 
+	if (hComOut == NULL)
+	{
+		Dprintf(("RS232: Failed to open output file %s\n",
+		         ConfigureParams.RS232.szOutFileName));
+		return FALSE;
+	}
+	setvbuf(hComOut, NULL, _IONBF, 0);
+
+	/* Create our COM file for input */
+	hComIn = fopen(ConfigureParams.RS232.szInFileName, "rb"); 
+	if (hComIn == NULL)
+	{
+		Dprintf(("RS232: Failed to open input file %s\n",
+		         ConfigureParams.RS232.szInFileName));
+		fclose(hComOut); hComOut = NULL;
+		return FALSE;
+	}
+	setvbuf(hComIn, NULL, _IONBF, 0);
+
+#if HAVE_TERMIOS_H
+	/* First set the output parameters to "raw" mode */
+	if (!RS232_SetRawMode(hComOut))
+	{
+		fprintf(stderr, "Can't set raw mode for %s\n",
+		        ConfigureParams.RS232.szOutFileName);
+	}
+
+	/* Now set the input parameters to "raw" mode */
+	if (!RS232_SetRawMode(hComIn))
+	{
+		fprintf(stderr, "Can't set raw mode for %s\n",
+		        ConfigureParams.RS232.szInFileName);
+	}
+#endif
+
+	/* Set all OK */
+	bConnectedRS232 = TRUE;
+
+	Dprintf(("Successfully opened RS232 files.\n"));
+
+	return TRUE;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Close file on COM port
+*/
+static void RS232_CloseCOMPort(void)
+{
+	/* Do have file open? */
+	if (bConnectedRS232)
+	{
+		bConnectedRS232 = FALSE;
+
+		/* Close */
+		fclose(hComIn);
+		hComIn = NULL;
+
+		fclose(hComOut);
+		hComOut = NULL;
+
+		Dprintf(("Closed RS232 files.\n"));
+	}
+}
+
+
+/* thread stuff */
+static SDL_sem* pSemFreeBuf;       /* Semaphore to sync free space in InputBuffer_RS232 */
+static SDL_Thread *RS232Thread = NULL; /* Thread handle for reading incoming data */
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Add incoming bytes from other machine into our input buffer
+*/
+static void RS232_AddBytesToInputBuffer(unsigned char *pBytes, int nBytes)
+{
+	int i;
+
+	/* Copy bytes into input buffer */
+	for (i=0; i<nBytes; i++)
+	{
+		SDL_SemWait(pSemFreeBuf);    /* Wait for free space in buffer */
+		InputBuffer_RS232[InputBuffer_Tail] = *pBytes++;
+		InputBuffer_Tail = (InputBuffer_Tail+1) % MAX_RS232INPUT_BUFFER;
+	}
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Thread to read incoming RS-232 data, and pass to emulator input buffer
+*/
+static int RS232_ThreadFunc(void *pData)
+{
+	int iInChar;
+	unsigned char cInChar;
+
+	/* Check for any RS-232 incoming data */
+	while (TRUE)
+	{
+		if (hComIn)
+		{
+			/* Read the bytes in, if we have any */
+			iInChar = fgetc(hComIn);
+			if (iInChar != EOF)
+			{
+				/* Copy into our internal queue */
+				cInChar = iInChar;
+				RS232_AddBytesToInputBuffer(&cInChar, 1);
+				/* FIXME: Use semaphores to lock MFP variables? */
+				MFP_InputOnChannel(MFP_RCVBUFFULL_BIT, MFP_IERA, &MFP_IPRA);
+				Dprintf(("RS232: Read character $%x\n", iInChar));
+			}
+			else
+			{
+				/*Dprintf(("RS232: Reached end of input file!\n"));*/
+				clearerr(hComIn);
+				SDL_Delay(20);
+			}
+
+			/* Sleep for a while */
+			SDL_Delay(2);
+		}
+		else
+		{
+			/* No RS-232 connection, sleep for 20ms */
+			SDL_Delay(20);
+		}
+	}
+
+	return(TRUE);
+}
+
 
 /*-----------------------------------------------------------------------*/
 /*
@@ -130,176 +367,6 @@ void RS232_UnInit(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  Set serial line parameters to "raw" mode.
-*/
-#if HAVE_TERMIOS_H
-static BOOL RS232_SetRawMode(FILE *fhndl)
-{
-	struct termios termmode;
-	int fd;
-
-	memset (&termmode, 0, sizeof(termmode));    /* Init with zeroes */
-	fd = fileno(fhndl);                         /* Get file descriptor */
-
-	if (isatty(fd))
-	{
-		if (tcgetattr(fd, &termmode) != 0)
-			return FALSE;
-
-		/* Set "raw" mode: */
-		termmode.c_cc[VMIN] = 1;
-		termmode.c_cc[VTIME] = 0;
-		cfmakeraw(&termmode);
-		if (tcsetattr(fd, TCSADRAIN, &termmode) != 0)
-			return FALSE;
-	}
-
-	return TRUE;
-}
-#endif /* HAVE_TERMIOS_H */
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Open file on COM port.
-*/
-BOOL RS232_OpenCOMPort(void)
-{
-	bConnectedRS232 = FALSE;
-
-	/* Create our COM file for output */
-	hComOut = fopen(ConfigureParams.RS232.szOutFileName, "wb"); 
-	if (hComOut == NULL)
-	{
-		Dprintf(("RS232: Failed to open output file %s\n",
-		         ConfigureParams.RS232.szOutFileName));
-		return FALSE;
-	}
-	setvbuf(hComOut, NULL, _IONBF, 0);
-
-	/* Create our COM file for input */
-	hComIn = fopen(ConfigureParams.RS232.szInFileName, "rb"); 
-	if (hComIn == NULL)
-	{
-		Dprintf(("RS232: Failed to open input file %s\n",
-		         ConfigureParams.RS232.szInFileName));
-		fclose(hComOut); hComOut = NULL;
-		return FALSE;
-	}
-	setvbuf(hComIn, NULL, _IONBF, 0);
-
-#if HAVE_TERMIOS_H
-	/* First set the output parameters to "raw" mode */
-	if (!RS232_SetRawMode(hComOut))
-	{
-		fprintf(stderr, "Can't set raw mode for %s\n",
-		        ConfigureParams.RS232.szOutFileName);
-	}
-
-	/* Now set the input parameters to "raw" mode */
-	if (!RS232_SetRawMode(hComIn))
-	{
-		fprintf(stderr, "Can't set raw mode for %s\n",
-		        ConfigureParams.RS232.szInFileName);
-	}
-#endif
-
-	/* Set all OK */
-	bConnectedRS232 = TRUE;
-
-	Dprintf(("Successfully opened RS232 files.\n"));
-
-	return TRUE;
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Close file on COM port
-*/
-void RS232_CloseCOMPort(void)
-{
-	/* Do have file open? */
-	if (bConnectedRS232)
-	{
-		bConnectedRS232 = FALSE;
-
-		/* Close */
-		fclose(hComIn);
-		hComIn = NULL;
-
-		fclose(hComOut);
-		hComOut = NULL;
-
-		Dprintf(("Closed RS232 files.\n"));
-	}
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Set hardware configuration of RS-232:
-  - Bits per character
-  - Parity
-  - Start/stop bits
-*/
-#if HAVE_TERMIOS_H
-static BOOL RS232_SetBitsConfig(int fd, int nCharSize, int nStopBits, BOOL bUseParity, BOOL bEvenParity)
-{
-	struct termios termmode;
-
-	memset (&termmode, 0, sizeof(termmode));    /* Init with zeroes */
-
-	if (isatty(fd))
-	{
-		if (tcgetattr(fd, &termmode) != 0)
-		{
-			Dprintf(("RS232_SetBitsConfig: tcgetattr failed.\n"));
-			return FALSE;
-		}
-
-		/* Set the character size: */
-		termmode.c_cflag &= ~CSIZE;
-		switch (nCharSize)
-		{
-		 case 8: termmode.c_cflag |= CS8; break;
-		 case 7: termmode.c_cflag |= CS7; break;
-		 case 6: termmode.c_cflag |= CS6; break;
-		 case 5: termmode.c_cflag |= CS5; break;
-		}
-
-		/* Set stop bits: */
-		if (nStopBits >= 2)
-			termmode.c_oflag |= CSTOPB;
-		else
-			termmode.c_oflag &= ~CSTOPB;
-
-		/* Parity bit: */
-		if (bUseParity)
-			termmode.c_cflag |= PARENB;
-		else
-			termmode.c_cflag &= ~PARENB;
-
-		if (bEvenParity)
-			termmode.c_cflag &= ~PARODD;
-		else
-			termmode.c_cflag |= PARODD;
-
-		/* Now store the configuration: */
-		if (tcsetattr(fd, TCSADRAIN, &termmode) != 0)
-		{
-			Dprintf(("RS232_SetBitsConfig: tcsetattr failed.\n"));
-			return FALSE;
-		}
-	}
-
-	return TRUE;
-}
-#endif /* HAVE_TERMIOS_H */
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Set hardware configuration of RS-232 according to the USART control register.
@@ -358,7 +425,7 @@ BOOL RS232_SetBaudRate(int nBaud)
 	int fd;
 	speed_t baudtype;
 	struct termios termmode;
-	static int baudtable[][2] =
+	static const int baudtable[][2] =
 	{
 		{ 50, B50 },
 		{ 75, B75 },
@@ -377,7 +444,9 @@ BOOL RS232_SetBaudRate(int nBaud)
 		{ 38400, B38400 },
 		{ 57600, B57600 },
 		{ 115200, B115200 },
+#ifdef B230400                 /* B230400 is not defined on all systems */
 		{ 230400, B230400 },
+#endif
 		{ -1, -1 }
 	};
 
@@ -572,76 +641,15 @@ BOOL RS232_GetStatus(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  Add incoming bytes from other machine into our input buffer
-*/
-static void RS232_AddBytesToInputBuffer(unsigned char *pBytes, int nBytes)
-{
-	int i;
-
-	/* Copy bytes into input buffer */
-	for (i=0; i<nBytes; i++)
-	{
-		SDL_SemWait(pSemFreeBuf);    /* Wait for free space in buffer */
-		InputBuffer_RS232[InputBuffer_Tail] = *pBytes++;
-		InputBuffer_Tail = (InputBuffer_Tail+1) % MAX_RS232INPUT_BUFFER;
-	}
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Thread to read incoming RS-232 data, and pass to emulator input buffer
-*/
-int RS232_ThreadFunc(void *pData)
-{
-	int iInChar;
-	unsigned char cInChar;
-
-	/* Check for any RS-232 incoming data */
-	while (TRUE)
-	{
-		if (hComIn)
-		{
-			/* Read the bytes in, if we have any */
-			iInChar = fgetc(hComIn);
-			if (iInChar != EOF)
-			{
-				/* Copy into our internal queue */
-				cInChar = iInChar;
-				RS232_AddBytesToInputBuffer(&cInChar, 1);
-				/* FIXME: Use semaphores to lock MFP variables? */
-				MFP_InputOnChannel(MFP_RCVBUFFULL_BIT, MFP_IERA, &MFP_IPRA);
-				Dprintf(("RS232: Read character $%x\n", iInChar));
-			}
-			else
-			{
-				/*Dprintf(("RS232: Reached end of input file!\n"));*/
-				clearerr(hComIn);
-				SDL_Delay(20);
-			}
-
-			/* Sleep for a while */
-			SDL_Delay(2);
-		}
-		else
-		{
-			/* No RS-232 connection, sleep for 20ms */
-			SDL_Delay(20);
-		}
-	}
-
-	return(TRUE);
-}
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Read from the Syncronous Character Register.
 */
 void RS232_SCR_ReadByte(void)
 {
+	M68000_WaitState(4);
+
+	/* nothing */
 }
 
 /*-----------------------------------------------------------------------*/
@@ -650,6 +658,8 @@ void RS232_SCR_ReadByte(void)
 */
 void RS232_SCR_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	/*Dprintf(("RS232: Write to SCR: $%x\n", (int)STRam[0xfffa27]));*/
 }
 
@@ -660,6 +670,8 @@ void RS232_SCR_WriteByte(void)
 */
 void RS232_UCR_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	Dprintf(("RS232: Read from UCR: $%x\n", (int)STRam[0xfffa29]));
 }
 
@@ -669,6 +681,8 @@ void RS232_UCR_ReadByte(void)
 */
 void RS232_UCR_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	Dprintf(("RS232: Write to UCR: $%x\n", (int)STRam[0xfffa29]));
 
 	if (bConnectedRS232)
@@ -682,6 +696,8 @@ void RS232_UCR_WriteByte(void)
 */
 void RS232_RSR_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	if (RS232_GetStatus())
 		STRam[0xfffa2b] |= 0x80;        /* Buffer full */
 	else
@@ -696,6 +712,8 @@ void RS232_RSR_ReadByte(void)
 */
 void RS232_RSR_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	Dprintf(("RS232: Write to RSR: $%x\n", (int)STRam[0xfffa2b]));
 }
 
@@ -706,6 +724,8 @@ void RS232_RSR_WriteByte(void)
 */
 void RS232_TSR_ReadByte(void)
 {
+	M68000_WaitState(4);
+
 	if (ConfigureParams.RS232.bEnableRS232)
 		STRam[0xfffa2d] |= 0x80;        /* Buffer empty */
 	else
@@ -720,6 +740,8 @@ void RS232_TSR_ReadByte(void)
 */
 void RS232_TSR_WriteByte(void)
 {
+	M68000_WaitState(4);
+
 	Dprintf(("RS232: Write to TSR: $%x\n", (int)STRam[0xfffa2d]));
 }
 
@@ -732,6 +754,8 @@ void RS232_UDR_ReadByte(void)
 {
 	unsigned char InByte;
 
+	M68000_WaitState(4);
+
 	RS232_ReadBytes(&InByte, 1);
 	STRam[0xfffa2f] = InByte;
 	Dprintf(("RS232: Read from UDR: $%x\n", (int)STRam[0xfffa2f]));
@@ -751,6 +775,8 @@ void RS232_UDR_WriteByte(void)
 {
 	unsigned char OutByte;
 
+	M68000_WaitState(4);
+
 	OutByte = STRam[0xfffa2f];
 	RS232_TransferBytesTo(&OutByte, 1);
 	Dprintf(("RS232: Write to UDR: $%x\n", (int)STRam[0xfffa2f]));
