@@ -19,7 +19,7 @@
   only convert the screen every 50 times a second - inbetween frames are not
   processed.
 */
-char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.45 2005/10/04 09:45:14 thothy Exp $";
+const char Screen_rcsid[] = "Hatari $Id: screen.c,v 1.50 2006/06/26 23:03:09 thothy Exp $";
 
 #include <SDL.h>
 #include <SDL_endian.h>
@@ -47,7 +47,7 @@ int STRes=ST_LOW_RES;         /* current resolution */
 int PrevSTRes=ST_LOW_RES;     /* previous ST resolution */
 Uint32 STRGBPalette[16];      /* Palette buffer used in conversion routines */
 Uint32 ST2RGB[4096];          /* Table to convert ST 0x777 / STe 0xfff palette to PC format RGB551 (2 pixels each entry) */
-SDL_Surface *sdlscrn;         /* The SDL screen surface */
+SDL_Surface *sdlscrn = NULL;  /* The SDL screen surface */
 Uint8 *pSTScreen;
 FRAMEBUFFER *pFrameBuffer;    /* Pointer into current 'FrameBuffer' */
 
@@ -61,8 +61,8 @@ static int STScreenWidthBytes;
 static int STScreenLineOffset[NUM_VISIBLE_LINES];  /* Offsets for ST screen lines eg, 0,160,320... */
 static Uint16 HBLPalette[16], PrevHBLPalette[16];  /* Current palette for line, also copy of first line */
 
-static void *ScreenDrawFunctionsNormal[4];         /* Screen draw functions */
-static void *ScreenDrawFunctionsVDI[3] =
+static void (*ScreenDrawFunctionsNormal[4])(void); /* Screen draw functions */
+static void (*ScreenDrawFunctionsVDI[3])(void) =
 {
   ConvertVDIRes_16Colour,
   ConvertVDIRes_4Colour,
@@ -76,38 +76,18 @@ static int ScrUpdateFlag;               /* Bit mask of how to update screen */
 
 /*-----------------------------------------------------------------------*/
 /*
-  Create ST 0x777 or STe 0xfff colour format to 16-bits per pixel.
-  Called each time when changed resolution or to/from fullscreen mode.
+  Create ST 0x777 / STe 0xfff color format to 16-bits per pixel conversion
+  table. Called each time when changed resolution or to/from fullscreen mode.
 */
 static void Screen_SetupRGBTable(void)
 {
   Uint16 STColour, RGBColour;
-  Uint16 r, g, b;
+  int r, g, b;
+  int rr, gg, bb;
 
-  if (ConfigureParams.System.nMachineType == MACHINE_ST)
+  /* Do Red, Green and Blue for all 16*16*16 = 4096 STe colors */
+  for(r=0; r < 16; r++)
   {
-    /* Do Red, Green and Blue for all 8*8*8 = 512 ST colours */
-    for(r=0; r < 8; r++)
-    {
-      for(g=0; g < 8; g++)
-      {
-        for(b=0; b < 8; b++)
-        {
-          /* ST 0x777 format */
-          STColour = (r<<8) | (g<<4) | (b);
-          RGBColour = SDL_MapRGB(sdlscrn->format, (r<<5), (g<<5), (b<<5));
-          /* As longs, for speed (write two pixels at once) */
-          ST2RGB[STColour] = (RGBColour<<16) | RGBColour;
-        }
-      }
-    }
-  }
-  else
-  {
-    int rr, gg, bb;
-    /* Do Red, Green and Blue for all 16*16*16 = 4096 STe colours */
-    for(r=0; r < 16; r++)
-    {
       for(g=0; g < 16; g++)
       {
         for(b=0; b < 16; b++)
@@ -122,7 +102,6 @@ static void Screen_SetupRGBTable(void)
           ST2RGB[STColour] = (RGBColour<<16) | RGBColour;
         }
       }
-    }
   }
 }
 
@@ -156,24 +135,6 @@ static void Screen_CreatePalette(void)
   }
   else
   {
-    if (ConfigureParams.System.nMachineType == MACHINE_ST)
-    {
-      /* Colors for ST color screen mode emulation */
-      for (i=0; i<16; i++)
-      {
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-        j = endiantable[i];
-#else
-        j = i;
-#endif
-        sdlColors[j].r = ((HBLPalettes[i]>>8) & 0x7) << 5;
-        sdlColors[j].g = ((HBLPalettes[i]>>4) & 0x7) << 5;
-        sdlColors[j].b = ( HBLPalettes[i]     & 0x7) << 5;
-      }
-      SDL_SetColors(sdlscrn, sdlColors, 10, 16);
-    }
-    else
-    {
       int r, g, b;
       /* Colors for STe color screen mode emulation */
       for (i=0; i<16; i++)
@@ -193,7 +154,6 @@ static void Screen_CreatePalette(void)
         sdlColors[j].b = (b & 0xe0) | ((b & 0x100) >> 4);
       }
       SDL_SetColors(sdlscrn, sdlColors, 10, 16);
-    }
   }
 }
 
@@ -345,6 +305,14 @@ static void Screen_SetResolution(void)
     sdlVideoFlags  = SDL_SWSURFACE|SDL_HWPALETTE;
   }
 
+  /* Check if we really have to change the video mode: */
+  if (sdlscrn && sdlscrn->w == Width && sdlscrn->h == Height
+      && sdlscrn->format->BitsPerPixel == BitCount
+      && (sdlscrn->flags&SDL_FULLSCREEN) == (sdlVideoFlags&SDL_FULLSCREEN))
+  {
+    return;
+  }
+
   sdlscrn = SDL_SetVideoMode(Width, Height, BitCount, sdlVideoFlags);
   if (!sdlscrn)
   {
@@ -821,7 +789,7 @@ static void Screen_Blit(BOOL bSwapScreen)
 /*     { 0,0, 640,400 },                                       /\* ST_HIGH_RES *\/ */
 /*     { (OVERSCAN_LEFT<<1),(OVERSCAN_BOTTOM<<1), 640,400 },   /\* ST_LOWMEDIUM_MIX_RES *\/ */
 /*   }; */
-  static SDL_Rect SrcWindowBitmapSizes[] =
+  static const SDL_Rect SrcWindowBitmapSizes[] =
   {
     { 0,0, 320,200 },                /* ST_LOW_RES */
     { 0,0, 640,400 },      /* ST_MEDIUM_RES */
@@ -829,7 +797,7 @@ static void Screen_Blit(BOOL bSwapScreen)
     { 0,0, 640,400 },   /* ST_LOWMEDIUM_MIX_RES */
   };
 
-  static SDL_Rect SrcWindowOverscanBitmapSizes[] =
+  static const SDL_Rect SrcWindowOverscanBitmapSizes[] =
   {
     { 0,0, OVERSCAN_LEFT+320+OVERSCAN_RIGHT,OVERSCAN_TOP+200+OVERSCAN_BOTTOM },
     { 0,0, (OVERSCAN_LEFT<<1)+640+(OVERSCAN_RIGHT<<1),(OVERSCAN_TOP<<1)+400+(OVERSCAN_BOTTOM<<1) },
@@ -838,7 +806,7 @@ static void Screen_Blit(BOOL bSwapScreen)
   };
 
   unsigned char *pTmpScreen;
-  SDL_Rect *SrcRect;
+  const SDL_Rect *SrcRect;
 
   /* Blit to full screen or window? */
   if (bInFullScreen)
@@ -883,7 +851,7 @@ static void Screen_Blit(BOOL bSwapScreen)
 */
 static void Screen_DrawFrame(BOOL bForceFlip)
 {
-  void *pDrawFunction;
+  void (*pDrawFunction)(void);
 
   /* Scan palette/resolution masks for each line and build up palette/difference tables */
   STRes = Screen_ComparePaletteMask();
