@@ -19,12 +19,13 @@
   sound and it simply doesn't work. If the emulator cannot keep the speed, users will have to turn off
   the sound - that's it.
 */
-char Sound_rcsid[] = "Hatari $Id: sound.c,v 1.21 2005/09/27 08:53:50 thothy Exp $";
+const char Sound_rcsid[] = "Hatari $Id: sound.c,v 1.26 2006/08/01 09:23:05 thothy Exp $";
 
 #include <SDL_types.h>
 
 #include "main.h"
 #include "audio.h"
+#include "cycles.h"
 #include "dmaSnd.h"
 #include "file.h"
 #include "int.h"
@@ -76,7 +77,6 @@ BOOL bEnvelopeFreqFlag;                                         /* As above, but
 /* Buffer to store circular samples */
 Sint8 MixBuffer[MIXBUFFER_SIZE];
 int nGeneratedSamples;                                          /* Generated samples since audio buffer update */
-int SoundCycles;
 
 
 /*-----------------------------------------------------------------------*/
@@ -87,7 +87,7 @@ typedef struct
 } ENVSHAPE;
 
 /* Envelope shapes */
-static ENVSHAPE EnvShapes[16] =
+static const ENVSHAPE EnvShapes[16] =
 {
  { {127,-128,-128,-128},    {-1, 0, 0, 0} },  /*  \_____  00xx  */
  { {127,-128,-128,-128},    {-1, 0, 0, 0} },  /*  \_____  00xx  */
@@ -108,7 +108,7 @@ static ENVSHAPE EnvShapes[16] =
 };
 
 /* Square wave look up table */
-static int SquareWave[16] = { 127,127,127,127,127,127,127,127, -128,-128,-128,-128,-128,-128,-128,-128 };
+static const int SquareWave[16] = { 127,127,127,127,127,127,127,127, -128,-128,-128,-128,-128,-128,-128,-128 };
 /* LogTable */
 static int LogTable[256];
 static int LogTable16[16];
@@ -151,7 +151,7 @@ static void Sound_CreateLogTables(void)
   Create envelope shape, store to table
   ( Wave is stored as 4 cycles, where cycles 1,2 are start and 3,4 are looped )
 */
-static void Sound_CreateEnvelopeShape(ENVSHAPE *pEnvShape,int *pEnvelopeValues)
+static void Sound_CreateEnvelopeShape(const ENVSHAPE *pEnvShape,int *pEnvelopeValues)
 {
   int i,j,Value;
 
@@ -230,14 +230,15 @@ void Sound_Reset(void)
   memset(MixBuffer, 0, MIXBUFFER_SIZE);
 
   /* Clear cycle counts, buffer index and register '13' flags */
-  SoundCycles = 0;
+  Cycles_SetCounter(CYCLES_COUNTER_SOUND, 0);
   bEnvelopeFreqFlag = FALSE;
   bWriteEnvelopeFreq = FALSE;
   bWriteChannelAAmp = bWriteChannelBAmp = bWriteChannelCAmp = FALSE;
 
   CompleteSndBufIdx = 0;
-  ActiveSndBufIdx =  (SoundBufferSize + SAMPLES_PER_FRAME) % MIXBUFFER_SIZE;
-  nGeneratedSamples = 0;
+  /* We do not start with 0 here to fake some initial samples: */
+  nGeneratedSamples = SoundBufferSize + SAMPLES_PER_FRAME;
+  ActiveSndBufIdx = nGeneratedSamples % MIXBUFFER_SIZE;
 
   /* Clear frequency counter */
   for(i=0; i<3; i++)
@@ -258,8 +259,8 @@ void Sound_Reset(void)
 void Sound_ResetBufferIndex(void)
 {
   Audio_Lock();
-  ActiveSndBufIdx =  (CompleteSndBufIdx + SoundBufferSize + SAMPLES_PER_FRAME)
-                     % MIXBUFFER_SIZE;
+  nGeneratedSamples = SoundBufferSize + SAMPLES_PER_FRAME;
+  ActiveSndBufIdx =  (CompleteSndBufIdx + nGeneratedSamples) % MIXBUFFER_SIZE;
   Audio_Unlock();
 }
 
@@ -280,19 +281,22 @@ void Sound_MemorySnapShot_Capture(BOOL bSave)
 /*-----------------------------------------------------------------------*/
 /*
   Find how many samples to generate and store in 'nSamplesToGenerate'
-  Also update 'SoundCycles' to store how many we actually did so generates set
-  amount each frame.
+  Also update sound cycles counter to store how many we actually did
+  so generates set amount each frame.
 */
 static void Sound_SetSamplesPassed(void)
 {
   int nSampleCycles;
   int nSamplesPerFrame;
   int Dec=1;
+  int nSoundCycles;
+
+  nSoundCycles = Cycles_GetCounter(CYCLES_COUNTER_SOUND);
 
   /* Check how many cycles have passed, as we use this to help find out if we are playing sample data */
 
   /* First, add decay to channel amplitude variables */
-  if (SoundCycles>(CYCLES_PER_FRAME/4))
+  if (nSoundCycles > (CYCLES_PER_FRAME/4))
     Dec = 16;                            /* Been long time between sound writes, must be normal tone sound */
 
   if (!bWriteChannelAAmp)                /* Not written to amplitude, decay value */
@@ -314,21 +318,21 @@ static void Sound_SetSamplesPassed(void)
   /* 160256 cycles per VBL, 44Khz = 882 samples per VBL */
   /* 882/160256 samples per clock cycle */
   nSamplesPerFrame = SAMPLES_PER_FRAME;
-#if 0  /* Use floats for calculation */
-  nSamplesToGenerate = (int)( (float)SoundCycles * ((float)nSamplesPerFrame/(float)CYCLES_PER_FRAME) );
-  if (nSamplesToGenerate > nSamplesPerFrame)
-    nSamplesToGenerate = nSamplesPerFrame;
 
-  nSampleCycles = (int)( (float)nSamplesToGenerate / ((float)nSamplesPerFrame/(float)CYCLES_PER_FRAME) );
-  SoundCycles -= nSampleCycles;
-#else  /* Use integers for calculation - both of these calculations should fit into 32-bit int */
-  nSamplesToGenerate = SoundCycles * nSamplesPerFrame / CYCLES_PER_FRAME;
+  nSamplesToGenerate = nSoundCycles * nSamplesPerFrame / CYCLES_PER_FRAME;
   if (nSamplesToGenerate > nSamplesPerFrame)
     nSamplesToGenerate = nSamplesPerFrame;
 
   nSampleCycles = nSamplesToGenerate * CYCLES_PER_FRAME / nSamplesPerFrame;
-  SoundCycles -= nSampleCycles;
-#endif
+  nSoundCycles -= nSampleCycles;
+  Cycles_SetCounter(CYCLES_COUNTER_SOUND, nSoundCycles);
+
+  if (nSamplesToGenerate > MIXBUFFER_SIZE - nGeneratedSamples)
+  {
+    nSamplesToGenerate = MIXBUFFER_SIZE - nGeneratedSamples;
+    if (nSamplesToGenerate < 0)
+      nSamplesToGenerate = 0;
+  }
 }
 
 
@@ -550,24 +554,6 @@ void Sound_Update_VBL(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  This is called from the audio callback function to create enough samples
-  to fill the current sound buffer.
-*/
-void Sound_UpdateFromAudioCallBack(void)
-{
-  /* If there are already enough samples or if we are recording, we should
-   * not generate more samples here! */
-  if(nGeneratedSamples >= SoundBufferSize || Sound_AreWeRecording())
-    return;
-
-  nSamplesToGenerate = SoundBufferSize - nGeneratedSamples;
-
-  Sound_GenerateSamples();
-}
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Start recording sound, as .YM or .WAV output
