@@ -26,12 +26,11 @@ int intlev (void);
 void check_prefs_changed_cpu(int new_level, int new_compatible);
 
 unsigned long OpCode_GemDos(uae_u32 opcode);
-unsigned long OpCode_OldGemDos(uae_u32 opcode);
 unsigned long OpCode_SysInit(uae_u32 opcode);
 unsigned long OpCode_VDI(uae_u32 opcode);
 
 
-#define write_log
+#define write_log printf
 
 
 #endif /* HATARI_GLUE_H */
