@@ -10,7 +10,7 @@
  * This file is distributed under the GNU Public License, version 2 or at
  * your option any later version. Read the file gpl.txt for details.
  */
-char ReadCpu_rcsid[] = "Hatari $Id: readcpu.c,v 1.5 2005/03/07 23:15:50 thothy Exp $";
+const char ReadCpu_rcsid[] = "Hatari $Id: readcpu.c,v 1.7 2006/02/09 22:02:26 eerot Exp $";
 
 #include <ctype.h>
 #include <string.h>
@@ -19,7 +19,7 @@ char ReadCpu_rcsid[] = "Hatari $Id: readcpu.c,v 1.5 2005/03/07 23:15:50 thothy E
 
 int nr_cpuop_funcs;
 
-struct mnemolookup lookuptab[] = {
+const struct mnemolookup lookuptab[] = {
     { i_ILLG, "ILLEGAL" },
     { i_OR, "OR" },
     { i_CHK, "CHK" },
