@@ -11,7 +11,7 @@
   We need to intercept the initial Line-A call (which we force into the TOS on
   boot-up) and also the init calls to the VDI.
 */
-char VDI_rcsid[] = "Hatari $Id: vdi.c,v 1.14 2005/07/15 19:30:32 thothy Exp $";
+const char VDI_rcsid[] = "Hatari $Id: vdi.c,v 1.17 2006/02/13 21:18:01 eerot Exp $";
 
 #include "main.h"
 #include "file.h"
@@ -33,16 +33,16 @@ int VDIWidth=640,VDIHeight=480;    /* 640x480,800x600 or 1024x768 */
 int VDIRes=0;                      /* 0,1 or 2(low, medium, high) */
 int VDIPlanes=4,VDIColours=16,VDICharHeight=8;  /* To match VDIRes */
 
-Uint32 Control;
-Uint32 Intin;
-Uint32 Ptsin;
-Uint32 Intout;
-Uint32 Ptsout;
+static Uint32 Control;
+static Uint32 Intin;
+static Uint32 Ptsin;
+static Uint32 Intout;
+static Uint32 Ptsout;
 
 
 /*-----------------------------------------------------------------------*/
 /* Desktop TOS 1.04 and TOS 2.06 desktop configuration files */
-unsigned char DesktopScript[504] =
+static const Uint8 DesktopScript[504] =
 {
 0x23,0x61,0x30,0x30,0x30,0x30,0x30,0x30,0x0D,0x0A,0x23,0x62,0x30,0x30,0x30,0x30,0x30,0x30,0x0D,0x0A,0x23,0x63,0x37,0x37,0x37,0x30,0x30,0x30,0x37,0x30,0x30,0x30,
 0x36,0x30,0x30,0x30,0x37,0x30,0x30,0x35,0x35,0x32,0x30,0x30,0x35,0x30,0x35,0x35,0x35,0x32,0x32,0x32,0x30,0x37,0x37,0x30,0x35,0x35,0x37,0x30,0x37,0x35,0x30,0x35,
@@ -62,7 +62,7 @@ unsigned char DesktopScript[504] =
 0x0A,0x23,0x46,0x20,0x30,0x33,0x20,0x30,0x34,0x20,0x20,0x20,0x2A,0x2E,0x54,0x4F,0x53,0x40,0x20,0x40,0x20,0x0D,0x0A,0x1A,
 };
 
-unsigned char NewDeskScript[786] =
+static const Uint8 NewDeskScript[786] =
 {
 0x23,0x61,0x30,0x30,0x30,0x30,0x30,0x30,0x0D,0x0A,0x23,0x62,0x30,0x30,0x30,0x30,0x30,0x30,0x0D,0x0A,0x23,0x63,0x37,0x37,0x37,0x30,0x30,0x30,0x37,0x30,0x30,0x30,
 0x36,0x30,0x30,0x30,0x37,0x30,0x30,0x35,0x35,0x32,0x30,0x30,0x35,0x30,0x35,0x35,0x35,0x32,0x32,0x32,0x30,0x37,0x37,0x30,0x35,0x35,0x37,0x30,0x37,0x35,0x30,0x35,
@@ -233,7 +233,7 @@ void VDI_Complete(void)
 /*
   Save desktop configuration file for VDI, eg desktop.inf(TOS 1.04) or newdesk.inf(TOS 2.06)
 */
-static void VDI_SaveDesktopInf(char *pszFileName,unsigned char *Script,long ScriptSize)
+static void VDI_SaveDesktopInf(char *pszFileName, const Uint8 *Script, long ScriptSize)
 {
   /* Just save file */
   File_Save(pszFileName, Script, ScriptSize, FALSE);
@@ -247,11 +247,11 @@ static void VDI_SaveDesktopInf(char *pszFileName,unsigned char *Script,long Scri
 static void VDI_ModifyDesktopInf(char *pszFileName)
 {
   long InfSize;
-  unsigned char *pInfData;
+  Uint8 *pInfData;
   int i;
 
   /* Load our '.inf' file */
-  pInfData = (unsigned char *)File_Read(pszFileName,NULL,&InfSize,NULL);
+  pInfData = (Uint8 *)File_Read(pszFileName,NULL,&InfSize,NULL);
   if (pInfData)
   {
     /* Scan file for '#E' */
