@@ -9,13 +9,13 @@
   TV raster trace, border removal, palette changes per HBL, the 'video address
   pointer' etc...
 */
-char Video_rcsid[] = "Hatari $Id: video.c,v 1.41 2005/09/01 19:20:49 eerot Exp $";
+const char Video_rcsid[] = "Hatari $Id: video.c,v 1.52 2006/08/09 08:14:24 eerot Exp $";
 
-#include <SDL.h>
 #include <SDL_endian.h>
 
 #include "main.h"
 #include "configuration.h"
+#include "cycles.h"
 #include "fdc.h"
 #include "int.h"
 #include "ikbd.h"
@@ -37,11 +37,12 @@ char Video_rcsid[] = "Hatari $Id: video.c,v 1.41 2005/09/01 19:20:49 eerot Exp $
 #define BORDERMASK_NONE    0x00                 /* Borders masks */
 #define BORDERMASK_LEFT    0x01
 #define BORDERMASK_RIGHT   0x02
+#define BORDERMASK_MIDDLE  0x04
 
 
 BOOL bUseHighRes = FALSE;                       /* Use hi-res (ie Mono monitor) */
 int nVBLs, nHBL;                                /* VBL Counter, HBL line */
-int nStartHBL, nEndHBL;                         /* Start/End HBL for visible screen(64 lines in Top border) */
+int nStartHBL, nEndHBL;                         /* Start/End HBL for visible screen */
 int OverscanMode;                               /* OVERSCANMODE_xxxx for current display frame */
 Uint16 HBLPalettes[(NUM_VISIBLE_LINES+1)*16];   /* 1x16 colour palette per screen line, +1 line just incase write after line 200 */
 Uint16 *pHBLPalettes;                           /* Pointer to current palette lists, one per HBL */
@@ -50,11 +51,17 @@ Uint32 *pHBLPaletteMasks;
 int nScreenRefreshRate = 50;                    /* 50 or 60 Hz in color, 70 Hz in mono */
 Uint32 VideoBase;                               /* Base address in ST Ram for screen (read on each VBL) */
 
+int nScanlinesPerFrame = 313;                   /* Number of scan lines per frame */
+int nCyclesPerLine = 512;                       /* Cycles per horizontal line scan */
+static int nFirstVisibleHbl = 34;               /* The first line of the ST screen that is copied to the PC screen buffer */
+
 static Uint8 HWScrollCount;                     /* HW scroll pixel offset, STe only (0...15) */
 static Uint8 ScanLineSkip;                      /* Scan line width add, STe only (words, minus 1) */
 static Uint8 *pVideoRaster;                     /* Pointer to Video raster, after VideoBase in PC address space. Use to copy data on HBL */
 static Uint8 VideoShifterByte;                  /* VideoShifter (0xff8260) value store in video chip */
 static int LeftRightBorder;                     /* BORDERMASK_xxxx used to simulate left/right border removal */
+static int nLastAccessCycleLeft;                /* Line cycle where program tried to open left border */
+static BOOL bSteBorderFlag;                     /* TRUE when screen width has been switched to 336 (e.g. in Obsession) */
 
 
 /*-----------------------------------------------------------------------*/
@@ -77,6 +84,10 @@ void Video_MemorySnapShot_Capture(BOOL bSave)
   MemorySnapShot_Store(&ScanLineSkip,sizeof(ScanLineSkip));
   MemorySnapShot_Store(&HWScrollCount,sizeof(HWScrollCount));
   MemorySnapShot_Store(&pVideoRaster,sizeof(pVideoRaster));
+  MemorySnapShot_Store(&nScanlinesPerFrame, sizeof(nScanlinesPerFrame));
+  MemorySnapShot_Store(&nCyclesPerLine, sizeof(nCyclesPerLine));
+  MemorySnapShot_Store(&nFirstVisibleHbl, sizeof(nFirstVisibleHbl));
+  MemorySnapShot_Store(&bSteBorderFlag, sizeof(bSteBorderFlag));
 }
 
 
@@ -90,10 +101,10 @@ static Uint32 Video_CalculateAddress(void)
   Uint32 VideoAddress;      /* Address of video display in ST screen space */
 
   /* Find number of cycles passed during frame */
-  FrameCycles = Int_FindFrameCycles();
+  FrameCycles = Cycles_GetCounterOnReadAccess(CYCLES_COUNTER_VIDEO);
 
-  /* Top of screen is usually 64 lines from VBL (64x512=32768 cycles) */
-  if (FrameCycles < nStartHBL*CYCLES_PER_LINE)
+  /* Top of screen is usually 63 lines from VBL in 50 Hz */
+  if (FrameCycles < nStartHBL*nCyclesPerLine)
   {
     VideoAddress = VideoBase;
   }
@@ -101,17 +112,17 @@ static Uint32 Video_CalculateAddress(void)
   {
     /* Now find which pixel we are on (ignore left/right borders) */
     /* 96 + 320 + 96 = 512 pixels per scan line (each pixel is one cycle) */
-    X = FrameCycles % CYCLES_PER_LINE;
+    X = FrameCycles % nCyclesPerLine;
     if (X < SCREEN_START_CYCLE)       /* Limit if in NULL area outside screen */
       X = SCREEN_START_CYCLE;
-    if (X > (CYCLES_PER_LINE - SCREEN_START_CYCLE))
-      X = (CYCLES_PER_LINE - SCREEN_START_CYCLE);
+    if (X > (nCyclesPerLine - SCREEN_START_CYCLE))
+      X = (nCyclesPerLine - SCREEN_START_CYCLE);
     /* X is now from 96 to 416 (320 pixels), subtract 96 to give X pixel across screen! */
     X = ((X-SCREEN_START_CYCLE)>>1)&(~1);
 
     VideoAddress = pVideoRaster - STRam;
     /* Add line cycles if we have not reached end of screen yet: */
-    if (FrameCycles < nEndHBL*CYCLES_PER_LINE)
+    if (FrameCycles < nEndHBL*nCyclesPerLine)
       VideoAddress += X;
   }
 
@@ -129,9 +140,9 @@ void Video_InterruptHandler_HBL(void)
   /* Remove this interrupt from list and re-order */
   Int_AcknowledgeInterrupt();
 
-  /* Generate new Timer AB, if need to - there are 313 HBLs per frame */
-  if(nHBL < (SCANLINES_PER_FRAME-1))
-    Int_AddAbsoluteInterrupt(CYCLES_PER_LINE,INTERRUPT_VIDEO_HBL);
+  /* Generate new HBL, if need to - there are 313 HBLs per frame in 50 Hz */
+  if (nHBL < nScanlinesPerFrame-1)
+    Int_AddAbsoluteInterrupt(nCyclesPerLine, INTERRUPT_VIDEO_HBL);
 
   M68000_Exception(EXCEPTION_HBLANK);   /* Horizontal blank interrupt, level 2! */
 }
@@ -146,10 +157,10 @@ static void Video_WriteToShifter(Uint8 Byte)
   static int nLastHBL = -1, LastByte, nLastCycles;
   int nFrameCycles, nLineCycles;
   
-  nFrameCycles = Int_FindFrameCycles();
+  nFrameCycles = Cycles_GetCounterOnWriteAccess(CYCLES_COUNTER_VIDEO);
 
   /* We only care for cycle position in the actual screen line */
-  nLineCycles = nFrameCycles % CYCLES_PER_LINE;
+  nLineCycles = nFrameCycles % nCyclesPerLine;
 
   /*fprintf(stderr,"Shifter=0x%2.2X %d (%d) @ %d\n",
           Byte, nFrameCycles, nLineCycles, nHBL);*/
@@ -162,6 +173,7 @@ static void Video_WriteToShifter(Uint8 Byte)
       && nLineCycles <= 48 && nLineCycles-nLastCycles <= 16)
   {
     LeftRightBorder |= BORDERMASK_LEFT;
+    nLastAccessCycleLeft = nLineCycles;
   }
 
   nLastHBL = nHBL;
@@ -183,10 +195,10 @@ void Video_Sync_WriteByte(void)
   /* Note: We're only interested in lower 2 bits (50/60Hz) */
   Byte = IoMem[0xff820a] & 3;
 
-  nFrameCycles = Int_FindFrameCycles();
+  nFrameCycles = Cycles_GetCounterOnWriteAccess(CYCLES_COUNTER_VIDEO);
 
   /* We only care for cycle position in the actual screen line */
-  nLineCycles = nFrameCycles % CYCLES_PER_LINE;
+  nLineCycles = nFrameCycles % nCyclesPerLine;
 
   /*fprintf(stderr,"Sync=0x%2.2X %d (%d) @ %d\n",
           Byte, nFrameCycles, nLineCycles, nHBL);*/
@@ -195,28 +207,39 @@ void Video_Sync_WriteByte(void)
    * FIXME: These are very inaccurate tests that should be improved,
    * but we probably need better CPU cycles emulation first. There's
    * also no support for sync-scrolling yet :-( */
-  if (LastByte == 0x00 && Byte == 0x02)   /* switched from 50 Hz to 60 Hz and back to 50 Hz? */
+  if (LastByte == 0x02 && Byte == 0x00)   /* switched from 50 Hz to 60 Hz? */
   {
-    if (nHBL >= OVERSCAN_TOP && nHBL <= 39 && nStartHBL > FIRST_VISIBLE_HBL)
+    if (nHBL >= SCREEN_START_HBL_60HZ-1 && nHBL <= SCREEN_START_HBL_60HZ+1)
     {
       /* Top border */
       OverscanMode |= OVERSCANMODE_TOP;       /* Set overscan bit */
-      nStartHBL = FIRST_VISIBLE_HBL;          /* New start screen line */
+      nStartHBL = SCREEN_START_HBL_60HZ;      /* New start screen line */
       pHBLPaletteMasks -= OVERSCAN_TOP;
       pHBLPalettes -= OVERSCAN_TOP;
     }
-    else if (nHBL == SCREEN_START_HBL+SCREEN_HEIGHT_HBL)
+    else if (nHBL == SCREEN_START_HBL_50HZ+SCREEN_HEIGHT_HBL)
     {
       /* Bottom border */
       OverscanMode |= OVERSCANMODE_BOTTOM;    /* Set overscan bit */
-      nEndHBL = SCREEN_START_HBL+SCREEN_HEIGHT_HBL+OVERSCAN_BOTTOM;  /* New end screen line */
+      nEndHBL = SCREEN_START_HBL_50HZ+SCREEN_HEIGHT_HBL+OVERSCAN_BOTTOM;  /* New end screen line */
     }
+  }
 
+  if (LastByte == 0x00 && Byte == 0x02)   /* switched from 60 Hz to 50 Hz? */
+  {
     if (nHBL == nLastHBL && nLineCycles >= 400 && nLineCycles <= 480
         && nLineCycles-nLastCycles <= 16)
     {
       /* Right border */
-      LeftRightBorder |= BORDERMASK_RIGHT;
+      //fprintf(stderr,"Right sync: %i - %i = %i\n", nLineCycles, nLastAccessCycleLeft, nLineCycles - nLastAccessCycleLeft);
+      if (nLineCycles - nLastAccessCycleLeft == 368)
+      {
+        LeftRightBorder |= BORDERMASK_MIDDLE;   /* Program tries to shorten line by 2 bytes */
+      }
+      else
+      {
+        LeftRightBorder |= BORDERMASK_RIGHT;    /* Program tries to open right border */
+      }
     }
   }
 
@@ -249,7 +272,7 @@ static void Video_StoreFirstLinePalette(void)
   for(i=0; i<16; i++)
     HBLPalettes[i] = SDL_SwapBE16(*pp2++);
   /* And set mask flag with palette and resolution */
-  HBLPaletteMasks[0] = (PALETTEMASK_RESOLUTION|PALETTEMASK_PALETTE) | (((unsigned long)IoMem_ReadByte(0xff8260)&0x3)<<16);
+  HBLPaletteMasks[0] = (PALETTEMASK_RESOLUTION|PALETTEMASK_PALETTE) | (((Uint32)IoMem_ReadByte(0xff8260)&0x3)<<16);
 }
 
 
@@ -263,7 +286,7 @@ static void Video_StoreResolution(int y)
   if (!(bUseHighRes || bUseVDIRes))
   {
     HBLPaletteMasks[y] &= ~(0x3<<16);
-    HBLPaletteMasks[y] |= ((unsigned long)IoMem_ReadByte(0xff8260)&0x3)<<16;
+    HBLPaletteMasks[y] |= ((Uint32)IoMem_ReadByte(0xff8260)&0x3)<<16;
   }
 }
 
@@ -338,7 +361,8 @@ static void Video_CopyScreenLineColor(void)
     /* Does have left border? If not, clear to color '0' */
     if (LeftRightBorder & BORDERMASK_LEFT)
     {
-      pVideoRaster += 24-SCREENBYTES_LEFT;
+      /* The "-2" in the following line is needed so that the offset is a multiple of 8 */
+      pVideoRaster += BORDERBYTES_LEFT-SCREENBYTES_LEFT-2;
       memcpy(pSTScreen, pVideoRaster, SCREENBYTES_LEFT);
       pVideoRaster += SCREENBYTES_LEFT;
     }
@@ -349,18 +373,35 @@ static void Video_CopyScreenLineColor(void)
     memcpy(pSTScreen+SCREENBYTES_LEFT, pVideoRaster, SCREENBYTES_MIDDLE);
     pVideoRaster += SCREENBYTES_MIDDLE;
 
-    /* Does have right border? If not, clear to color '0' */
+    /* Does have right border? */
     if (LeftRightBorder & BORDERMASK_RIGHT)
     {
       memcpy(pSTScreen+SCREENBYTES_LEFT+SCREENBYTES_MIDDLE, pVideoRaster, SCREENBYTES_RIGHT);
-      pVideoRaster += 46-SCREENBYTES_RIGHT;
+      pVideoRaster += BORDERBYTES_RIGHT-SCREENBYTES_RIGHT;
       pVideoRaster += SCREENBYTES_RIGHT;
     }
+    else if (LeftRightBorder & BORDERMASK_MIDDLE)
+    {
+      /* Shortened line by 2 bytes? */
+      memset(pSTScreen+SCREENBYTES_LEFT+SCREENBYTES_MIDDLE-2, 0, SCREENBYTES_RIGHT+2);
+      pVideoRaster -= 2;
+    }
     else
+    {
+      /* Simply clear right border to '0' */
       memset(pSTScreen+SCREENBYTES_LEFT+SCREENBYTES_MIDDLE,0,SCREENBYTES_RIGHT);
+    }
 
-    /* Handle STE fine scrolling (HWScrollCount is zero on ST). */
-    if (HWScrollCount)
+    /* Correct the "-2" offset for pVideoRaster from BORDERMASK_LEFT above */
+    if (LeftRightBorder & BORDERMASK_LEFT)
+      pVideoRaster += 2;
+
+    if (bSteBorderFlag)
+    {
+      memcpy(pSTScreen+SCREENBYTES_LEFT+SCREENBYTES_MIDDLE, pVideoRaster, 4*2);
+      pVideoRaster += 4 * 2;
+    }
+    else if (HWScrollCount)     /* Handle STE fine scrolling (HWScrollCount is zero on ST) */
     {
       Uint16 *pScrollAdj;       /* Pointer to actual position in line */
       int nNegScrollCnt;
@@ -437,13 +478,6 @@ static void Video_CopyScreenLineColor(void)
 */
 static void Video_CopyScreenLine(void)
 {
-  /* Read STE fine scrolling registers */
-  if (ConfigureParams.System.nMachineType != MACHINE_ST)
-  {
-    ScanLineSkip = STMemory_ReadByte(0xff820f);
-    HWScrollCount = STMemory_ReadByte(0xff8265) & 0x0f;
-  }
-
   /* Only copy screen line if not doing high VDI resolution */
   if (!bUseVDIRes)
   {
@@ -478,21 +512,45 @@ static void Video_CopyVDIScreen(void)
 */
 static void Video_EndHBL(void)
 {
+  Uint8 nSyncByte = IoMem_ReadByte(0xff820a);
+
+  /* Check if we need to open borders: If we are running basically in 50 Hz, but
+   * a program switched to 60 Hz at certain screen lines, we have to open the
+   * corresponding border. The "Level 16" fullscreen in the Union demo is a good example. */
+  if ((nSyncByte & 2) == 0)
+  {
+    if (nHBL == SCREEN_START_HBL_60HZ-1 && nStartHBL == SCREEN_START_HBL_50HZ)
+    {
+      /* Top border */
+      OverscanMode |= OVERSCANMODE_TOP;       /* Set overscan bit */
+      nStartHBL = SCREEN_START_HBL_60HZ;      /* New start screen line */
+      pHBLPaletteMasks -= OVERSCAN_TOP;
+      pHBLPalettes -= OVERSCAN_TOP;
+    }
+    else if (nHBL == SCREEN_START_HBL_50HZ+SCREEN_HEIGHT_HBL-1
+             && nEndHBL == SCREEN_START_HBL_50HZ+SCREEN_HEIGHT_HBL)
+    {
+      /* Bottom border */
+      OverscanMode |= OVERSCANMODE_BOTTOM;    /* Set overscan bit */
+      nEndHBL = SCREEN_START_HBL_50HZ+SCREEN_HEIGHT_HBL+OVERSCAN_BOTTOM;  /* New end screen line */
+    }
+  }
+
   /* Are we in possible visible display (including borders)? */
-  if (nHBL >= FIRST_VISIBLE_HBL && nHBL < FIRST_VISIBLE_HBL+NUM_VISIBLE_LINES)
+  if (nHBL >= nFirstVisibleHbl && nHBL < nFirstVisibleHbl+NUM_VISIBLE_LINES)
   {
     /* Copy line of screen to buffer to simulate TV raster trace
      * - required for mouse cursor display/game updates
      * Eg, Lemmings and The Killing Game Show are good examples */
     Video_CopyScreenLine();
 
-    if (nHBL == FIRST_VISIBLE_HBL)    /* Very first line on screen - HBLPalettes[0] */
+    if (nHBL == nFirstVisibleHbl)    /* Very first line on screen - HBLPalettes[0] */
     {
       /* Store ALL palette for this line into raster table for datum */
       Video_StoreFirstLinePalette();
     }
     /* Store resolution for every line so can check for mix low/medium screens */
-    Video_StoreResolution(nHBL-FIRST_VISIBLE_HBL);
+    Video_StoreResolution(nHBL-nFirstVisibleHbl);
   }
 
   /* Finally increase HBL count */
@@ -513,8 +571,8 @@ void Video_InterruptHandler_EndLine(void)
   /* Remove this interrupt from list and re-order */
   Int_AcknowledgeInterrupt();
   /* Generate new HBL, if need to - there are 313 HBLs per frame */
-  if (nHBL < SCANLINES_PER_FRAME-1)
-    Int_AddAbsoluteInterrupt(CYCLES_PER_LINE,INTERRUPT_VIDEO_ENDLINE);
+  if (nHBL < nScanlinesPerFrame-1)
+    Int_AddAbsoluteInterrupt(nCyclesPerLine, INTERRUPT_VIDEO_ENDLINE);
 
   /* Is this a good place to send the keyboard packets? Done once per frame */
   if (nHBL == nStartHBL)
@@ -567,18 +625,16 @@ static void Video_SetHBLPaletteMaskPointers(void)
   int FrameCycles;
   int Line;
 
-  /* Top of standard screen is 64 lines from VBL(64x512=32768 cycles) */
-  /* Each line is 64+320+64+64(Blank) = 512 pixels per scan line      */
-  /* Timer occurs at end of 64+320+64; Display Enable(DE)=Low         */
-  /* HBL is incorrect on machine and occurs around 96+ cycles in      */
+  /* Top of standard screen is 63 lines from VBL (in 50 Hz)      */
+  /* Each line is 64+320+64+64(Blank) = 512 pixels per scan line */
+  /* Timer occurs at end of 64+320+64; Display Enable(DE)=Low    */
+  /* HBL is incorrect on machine and occurs around 96+ cycles in */
 
-  /* Top of standard screen is 64 lines from VBL(64x512=32768 cycles) */
-  /* Each line is 96 + 320 + 96 = 512 pixels per scan line(each pixel is one cycle) */
-  FrameCycles = Int_FindFrameCycles();
+  FrameCycles = Cycles_GetCounter(CYCLES_COUNTER_VIDEO);
 
-  /* Find 'line' into palette - screen starts 64 lines down, less 28 for top overscan */
+  /* Find 'line' into palette - screen starts 63 lines down, less 29 for top overscan */
   /* And if write to last 96 cycle of line it will count as the NEXT line(needed else games may flicker) */
-  Line = (FrameCycles-(FIRST_VISIBLE_HBL*CYCLES_PER_LINE)+SCREEN_START_CYCLE)/CYCLES_PER_LINE;
+  Line = (FrameCycles-(nFirstVisibleHbl*nCyclesPerLine)+SCREEN_START_CYCLE)/nCyclesPerLine;
   if (Line<0)          /* Limit to top/bottom of possible visible screen */
     Line = 0;
   if (Line>=NUM_VISIBLE_LINES)
@@ -590,6 +646,48 @@ static void Video_SetHBLPaletteMaskPointers(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  Set video shifter timing variables according to screen refresh rate
+*/
+static void Video_ResetShifterTimings(void)
+{
+  Uint8 nSyncByte;
+
+  nSyncByte = IoMem_ReadByte(0xff820a);
+
+  /* Check if running in 50 Hz or in 60 Hz */
+  if (nSyncByte & 2)
+  {
+    /* 50 Hz */
+    nStartHBL = SCREEN_START_HBL_50HZ;
+    nScanlinesPerFrame = SCANLINES_PER_FRAME_50HZ;
+    nCyclesPerLine = CYCLES_PER_LINE_50HZ;
+    nFirstVisibleHbl = FIRST_VISIBLE_HBL_50HZ;
+  }
+  else
+  {
+    /* 60 Hz */
+    nStartHBL = SCREEN_START_HBL_60HZ;
+    nScanlinesPerFrame = SCANLINES_PER_FRAME_60HZ;
+    nCyclesPerLine = CYCLES_PER_LINE_60HZ;
+    nFirstVisibleHbl = FIRST_VISIBLE_HBL_60HZ;
+  }
+
+  nEndHBL = nStartHBL + SCREEN_HEIGHT_HBL;
+
+  /* Set the screen refresh rate */
+#if 0
+  if(bUseHighRes)
+    nScreenRefreshRate = 70;
+  else if (nSyncByte & 2)               /* Is it 50Hz or is it 60Hz? */
+    nScreenRefreshRate = 50;
+  else
+    nScreenRefreshRate = 60;
+#endif
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Called on VBL, set registers ready for frame
@@ -598,16 +696,16 @@ static void Video_ClearOnVBL(void)
 {
   /* New screen, so first HBL */
   nHBL = 0;
-  nStartHBL = SCREEN_START_HBL;
-  nEndHBL = SCREEN_START_HBL+SCREEN_HEIGHT_HBL;
   OverscanMode = OVERSCANMODE_NONE;
 
+  Video_ResetShifterTimings();
+
   /* Get screen address pointer, aligned to 256 bytes on ST (ie ignore lowest byte) */
-  VideoBase = (Uint32)STMemory_ReadByte(0xff8201)<<16 | (Uint32)STMemory_ReadByte(0xff8203)<<8;
+  VideoBase = (Uint32)IoMem_ReadByte(0xff8201)<<16 | (Uint32)IoMem_ReadByte(0xff8203)<<8;
   if (ConfigureParams.System.nMachineType != MACHINE_ST)
   {
     /* on STe 2 aligned, on Falcon 4 aligned, on TT 8 aligned. We do STe. */
-    VideoBase |= STMemory_ReadByte(0xff820d) & ~1;
+    VideoBase |= IoMem_ReadByte(0xff820d) & ~1;
   }
   pVideoRaster = &STRam[VideoBase];
   pSTScreen = pFrameBuffer->pSTScreen;
@@ -625,37 +723,24 @@ static void Video_ClearOnVBL(void)
 void Video_InterruptHandler_VBL(void)
 {
   int PendingCyclesOver;
-  int nNewMilliTicks;
-  static int nOldMilliTicks = 0;
-  signed int nDelay;
 
   /* Store cycles we went over for this frame(this is our inital count) */
   PendingCyclesOver = -PendingInterruptCount;    /* +ve */
 
   /* Remove this interrupt from list and re-order */
   Int_AcknowledgeInterrupt();
-  /* Start HBL interrupts - MUST do before add in cycles */
-  Int_AddAbsoluteInterrupt(CYCLES_ENDLINE,INTERRUPT_VIDEO_ENDLINE);
+  /* Start HBL interrupts */
+  Int_AddAbsoluteInterrupt(nCyclesPerLine, INTERRUPT_VIDEO_ENDLINE);
   Int_AddAbsoluteInterrupt(CYCLES_HBL,INTERRUPT_VIDEO_HBL);
   Int_AddAbsoluteInterrupt(CYCLES_PER_FRAME,INTERRUPT_VIDEO_VBL);
 
   /* Set frame cycles, used for Video Address */
-  nFrameCyclesOver = PendingCyclesOver;      /* Number of cycles into frame */
-
-  /* Set the screen refresh rate */
-#if 0
-  if(bUseHighRes)
-    nScreenRefreshRate = 70;
-  else if(IoMem[0xff820a] & 2)               /* Is it 50Hz or is it 60Hz? */
-    nScreenRefreshRate = 50;
-  else
-    nScreenRefreshRate = 60;
-#endif
+  Cycles_SetCounter(CYCLES_COUNTER_VIDEO, PendingCyclesOver);
 
   /* Clear any key presses which are due to be de-bounced (held for one ST frame) */
   Keymap_DebounceAllKeys();
-  /* Check shortcut keys */
-  ShortCut_CheckKeys();
+  /* Act on shortcut keys */
+  ShortCut_ActKey();
 
   /* Draw screen, skip frame if need to */
   if (!ConfigureParams.Screen.bFrameSkip || (nVBLs&1))
@@ -688,22 +773,7 @@ void Video_InterruptHandler_VBL(void)
     set_special(SPCFLAG_BRK);         /* Assure that CPU core shuts down */
   }
 
-  if (ConfigureParams.System.nMinMaxSpeed != MINMAXSPEED_MAX)
-  {
-    /* Wait, so we stay in sync with the sound */
-    do
-    {
-      nNewMilliTicks = SDL_GetTicks();
-      nDelay = 1000/nScreenRefreshRate - (nNewMilliTicks-nOldMilliTicks);
-      if(nDelay > 2)
-      {
-        /* SDL_Delay seems to be quite inaccurate, so we don't wait the whole time */
-        SDL_Delay(nDelay - 1);
-      }
-    }
-    while(nDelay > 0);
-    nOldMilliTicks = nNewMilliTicks;
-  }
+  Main_WaitOnVbl();
 }
 
 
@@ -730,6 +800,8 @@ void Video_Reset(void)
   /* Reset STe screen variables */
   ScanLineSkip = 0;
   HWScrollCount = 0;
+  bSteBorderFlag = FALSE;
+
   /* Clear ready for new VBL */
   Video_ClearOnVBL();
 }
@@ -818,6 +890,8 @@ void Video_LineWidth_ReadByte(void)
 {
   if (ConfigureParams.System.nMachineType == MACHINE_ST)
     IoMem[0xff820f] = 0;        /* On ST this is always 0 */
+  else
+    IoMem[0xff820f] = ScanLineSkip;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -832,8 +906,25 @@ void Video_ShifterMode_ReadByte(void)
     IoMem[0xff8260] = VideoShifterByte;   /* Read shifter register */
 }
 
+/*-----------------------------------------------------------------------*/
+/*
+  Read horizontal scroll register (0xff8265)
+*/
+void Video_HorScroll_Read(void)
+{
+  IoMem[0xff8265] = HWScrollCount;
+}
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  Write video line width register (0xff820f) - STE only.
+*/
+void Video_LineWidth_WriteByte(void)
+{
+    ScanLineSkip = IoMem_ReadByte(0xff820f);
+}
+
 /*-----------------------------------------------------------------------*/
 /*
   Write to video shifter palette registers (0xff8240-0xff825e)
@@ -952,6 +1043,46 @@ void Video_ShifterMode_WriteByte(void)
     Video_SetHBLPaletteMaskPointers();
     *pHBLPaletteMasks &= 0xff00ffff;
     /* Store resolution after palette mask and set resolution write bit: */
-    *pHBLPaletteMasks |= (((unsigned long)VideoShifterByte|0x04)<<16);
+    *pHBLPaletteMasks |= (((Uint32)VideoShifterByte|0x04)<<16);
   }
 }
+
+/*-----------------------------------------------------------------------*/
+/*
+  Write to horizontal scroll register (0xff8265).
+  Note: The STE shifter has a funny "bug"  that allows to increase the
+  resolution to 336x200 instead of 320x200. It occurs when a program writes
+  certain values to 0xff8264:
+	move.w  #1,$ffff8264      ; Word access!
+	clr.b   $ffff8264         ; Byte access!
+  Some games (Obsession, Skulls) and demos (Pacemaker by Paradox) use this
+  feature to increase the resolution, so we have to emulate this bug, too!
+*/
+void Video_HorScroll_Write(void)
+{
+  static BOOL bFirstSteAccess = FALSE;
+
+  HWScrollCount = IoMem[0xff8265];
+
+  /*fprintf(stderr, "Write to 0x%x (0x%x, 0x%x, %i)\n", IoAccessBaseAddress,
+          IoMem[0xff8264], HWScrollCount, nIoMemAccessSize);*/
+
+  if (IoAccessBaseAddress == 0xff8264 && nIoMemAccessSize == SIZE_WORD
+      && HWScrollCount == 1)
+  {
+    /*fprintf(stderr, "STE border removal - access 1\n");*/
+    bFirstSteAccess = TRUE;
+  }
+  else if (bFirstSteAccess && HWScrollCount == 1 &&
+           IoAccessBaseAddress == 0xff8264 && nIoMemAccessSize == SIZE_BYTE)
+  {
+    /*fprintf(stderr, "STE border removal - access 2\n");*/
+    bSteBorderFlag = TRUE;
+  }
+  else
+  {
+    bFirstSteAccess = bSteBorderFlag = FALSE;
+  }
+
+  HWScrollCount &= 0x0f;
+}
