@@ -9,7 +9,7 @@
   We intercept and direct some XBios calls to handle the RS-232 etc. and help
   with floppy debugging.
 */
-char XBios_rcsid[] = "Hatari $Id: xbios.c,v 1.9 2005/07/15 19:30:32 thothy Exp $";
+const char XBios_rcsid[] = "Hatari $Id: xbios.c,v 1.11 2006/02/12 21:28:22 eerot Exp $";
 
 #include "main.h"
 #include "configuration.h"
@@ -27,7 +27,7 @@ char XBios_rcsid[] = "Hatari $Id: xbios.c,v 1.9 2005/07/15 19:30:32 thothy Exp $
 
 
 /* List of Atari ST RS-232 baud rates */
-static int BaudRates[] = {
+static const int BaudRates[] = {
   19200, /* 0 */
   9600,  /* 1 */
   4800,  /* 2 */
